/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2004 Kenneth W. Sodemann (stuffle@mac.com)
 *-------------------------------------------------------------------------
 * project_dlg
 *
 * Synopsis:
 *   Handle the project listing and editing dialog boxes.
 *
 * $Id: project_dlg.c,v 1.75 2004/11/25 23:44:07 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <ctype.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gnome.h>
#include <gconf/gconf-client.h>
#include <libpq-fe.h>

#include "db_utils.h"
#include "defs.h"
#include "error_chks.h"
#include "gstr_utils.h"
#include "gtkutils.h"
#include "mainwin.h"
#include "prepsmain.h"
#include "project_dlg.h"
#include "util.h"

/*
 *  Define the names of the significant widgets
 */
#define PROJECT_DIALOG        "project_dlg"
#define PROJECT_LIST_VIEW     "project_list_view"
#define ADD_BUTTON            "add_button"
#define MOD_BUTTON            "modify_button"
#define DEL_BUTTON            "delete_button"
#define SHOW_INACTIVE_CB      "show_inactive_cb"
#define PROJECT_EDIT_DIALOG   "project_edit_dlg"
#define NAME_ENTRY            "name_entry"
#define ACTIVE_CB             "active_cb"
#define DESCR_TEXT            "descr_text"
#define LEADER_COMBO          "leader_combo"
#define USER_LIST_VIEW        "user_list_view"
#define SUBMITTER_LIST        "submitter_clist"
#define SUB_ADD_ALL_BTN       "sub_add_all_btn"
#define SUB_RM_ALL_BTN        "sub_rm_all_btn"
#define PRJ_MEMBER_LIST       "project_member_clist"
#define PRJ_MEM_ADD_ALL_BTN   "prjmem_add_all_btn"
#define PRJ_MEM_RM_ALL_BTN    "prjmem_rm_all_btn"
#define NEW_NOTIFY_GRP_CB     "new_pr_notify_grp"
#define NEW_NOTIFY_PL_CB      "new_pr_notify_pl"
#define ASSIGN_NOTIFY_RESP_CB "assign_notify_resp"
#define ASSIGN_NOTIFY_SUB_CB  "assign_notify_sub"
#define ASSIGN_NOTIFY_PL_CB   "assign_notify_pl"
#define UPD_NOTIFY_GRP_CB     "upd_notify_grp"
#define UPD_NOTIFY_PL_CB      "upd_notify_pl"
#define UPD_NOTIFY_RESP_CB    "upd_notify_resp"
#define UPD_NOTIFY_SUB_CB     "upd_notify_sub"
#define CHECK_BOX_ITEM        "check_box_item"
#define AUTO_ASSIGN_COMBO     "auto_assign_combo"
#define ADD_VER_BUTTON        "add_version_btn"
#define DEL_VER_BUTTON        "del_version_btn"
#define SHOW_ALL_VER_CB       "show_all_ver_cb"
#define SHOW_INACTIVE_USERS   "show_inactive_users"
#define VER_LIST_VIEW         "ver_list_view"
#define NAME_LIST             "name_list"
#define ID_LIST               "id_list"

#define COLUMN_NUM            "column_num"

/*
 * 'Null' version display data (for the combo boxes).
 */
#define NULL_NAME           _("<None>")
#define NULL_NAME_SIZE      6

#define AUTO_ASSIGN_NAME_FORMAT "%s (%s)"

/*
 *  This is the order in the main SELECT.  If the order changes in 
 *  read_project_data, these must change to match it.
 *  
 *  In order for stuff to work right, these queries MUST select
 *  stuff back in the same order.  This helps make the extraction
 *  functions more generic.
 */
#define PK_NUM_POS              0
#define ID_POS                  0
#define NAME_POS                1
#define ACTIVE_POS              2
#define LOGIN_ID_POS            3
#define DESCR_POS               4
#define LAST_NAME_POS           5
#define FIRST_NAME_POS          6
#define NEW_PR_GRP_POS          5
#define NEW_PR_PL_POS           6
#define ASSIGN_RESP_POS         7
#define ASSIGN_SUB_POS          8
#define ASSIGN_PL_POS           9
#define UPD_GRP_POS             10
#define UPD_PL_POS              11
#define UPD_RESP_POS            12
#define UPD_SUB_POS             13
#define OPEN_VER_POS            14
#define CLOSE_VER_POS           15
#define AUTO_ASSIGN_POS         16
#define ORDER_NUM_POS           3
#define PROJECT_QUERY "SELECT project_num, name, active, login_id, \
                              descr, new_pr_notify_grp, new_pr_notify_pl, \
                              assign_notify_resp, assign_notify_sub, \
                              assign_notify_pl, upd_notify_grp, \
                              upd_notify_pl, upd_notify_resp, upd_notify_sub,\
                              open_version, close_version, \
                              auto_assign_login_id \
                         FROM project \
                        WHERE project_num = %d "

#define PERSON_QUERY "SELECT 0, 1, active, login_id, 4, last_name, \
                             first_name \
                        FROM person " 
#define WHERE_PERSON_ACTIVE "WHERE active = TRUE "
#define PERSON_ORDER_BY "ORDER BY last_name, first_name, login_id "
#define LEADER_QUERY "SELECT 0, 1, active, login_id, 4, last_name, \
                             first_name \
                        FROM person \
                       WHERE active = TRUE \
                      UNION \
                      SELECT 0, 1, p.active, p.login_id, 4, p.last_name, \
                             p.first_name \
                        FROM project prj, person p \
                       WHERE p.login_id = prj.login_id \
                         AND prj.project_num = %d " 

#define VERSION_QUERY "SELECT version_num, version_text, active, \
                              order_num, descr \
                         FROM project_version \
                        WHERE project_num = %d "
#define WHERE_VERSION_ACTIVE "AND active = TRUE "
#define VERSION_ORDER_BY "ORDER BY order_num"      


/*
 *  Various queries used to insert, update, and remove data.
 */
#define RM_RESPONSIBLE_SQL "DELETE \
                              FROM responsible \
                             WHERE project_num = %d \
                               AND login_id = '%s'"
                               
#define RM_SUBMITTER_SQL "DELETE \
                            FROM submitter \
                           WHERE project_num = %d \
                             AND login_id = '%s'"
                             
#define INS_RESPONSIBLE_SQL "INSERT \
                               INTO responsible (project_num, login_id) \
                             VALUES (%d, '%s')"

#define INS_SUBMITTER_SQL "insert into submitter (project_num, login_id) \
                           select %d, login_id \
                             from person \
                            where login_id = '%s' \
                              and not exists (select 'x' \
                                                from submitter \
                                               where login_id = '%s' \
                                                 and project_num = %d)"

#define RESP_LIST_SQL "\
SELECT distinct r.login_id, \
       p.last_name || ', ' || p.first_name || ' (' || p.login_id || ')' \
  FROM responsible r, person p \
 WHERE p.login_id = r.login_id \
   AND r.project_num = %d \
 ORDER BY 2"

#define GET_USER_FULL_NAME "\
SELECT last_name || ', ' || first_name || ' (' || login_id || ')' \
  FROM person \
 WHERE login_id = '%s'"

#define UPDATE_PRJ_LEADER "\
UPDATE project \
   SET login_id = '%s' \
 WHERE project_num = %d"

#define UPDATE_AUTO_ASSIGN "\
UPDATE project \
   SET auto_assign_login_id = '%s' \
 WHERE project_num = %d "

#define UPDATE_AUTO_ASSIGN_NULL "\
UPDATE project \
   SET auto_assign_login_id = NULL \
 WHERE project_num = %d "

#define SUB_LIST_SQL "SELECT  distinct login_id \
                        FROM submitter \
                       WHERE project_num = %d"

#define INSERT_VERSION "\
INSERT \
  INTO project_version (version_num, project_num, version_text, descr, \
                        active, order_num) \
VALUES (%d, %d, %s, %s, %s, %d)"

#define UPDATE_VERSION "\
UPDATE project_version \
   SET version_text = '%s', \
       descr = '%s', \
       active = %s, \
       order_num = %d \
 WHERE version_num = %d \
   AND project_num = %d"

#define DELETE_VERSION "\
DELETE \
  FROM project_version \
 WHERE version_num = %d \
   and project_num = %d"

/*
 * Enumerations defining columns in specific list models.
 */
enum {
   PROJECT_NAME = 0,
   PROJECT_DESCR,
   PROJECT_ACTIVE,
   PROJECT_EDITABLE,
   PROJECT_PK,
   NUM_PROJECT_COLS
};

enum {
   PERSON_NAME = 0,
   PERSON_LOGIN_ID,
   PERSON_SUB,
   PERSON_RESP,
   NUM_PERSON_COLS
};

enum {
   VERSION_NAME = 0,
   VERSION_DESCR,
   VERSION_ACTIVE,
   VERSION_DEF_OPEN,
   VERSION_DEF_CLOSE,
   VERSION_NUM,
   NUM_VERSION_COLS
};


/*
 * Defines the toggle buttons tied directly to columns in the
 * project table.
 */
enum {
   NEW_NOTIFY_GRP = 1,
   NEW_NOTIFY_PL,
   ASSIGN_NOTIFY_RESP,
   ASSIGN_NOTIFY_SUB,
   ASSIGN_NOTIFY_PL,
   UPD_NOTIFY_GRP,
   UPD_NOTIFY_PL,
   UPD_NOTIFY_RESP,
   UPD_NOTIFY_SUB,
   PRJ_ACTIVE
};

static const gchar *my_child_list = "CHILD_LIST";
                             
/*
 * This application can only have one projet list dialog visible at any
 * given time.  This is that dialog...
 */
static GtkWidget  *project_dlg = NULL;

/*
 *  Local prototypes
 */
static gboolean draw_project_list (GtkWindow *parent, PGconn *conn);



/*----------------------------------------------------------------------------
 * General Utility Functions
 *
 * NOTE:
 * These functions should be placed in one of the util functions, but do
 * this AFTER we get all of the util functions moved from libPRepS into
 * the main body of code.
 *--------------------------------------------------------------------------*/
static gint
g_string_strcmp (const GString *str1, const gchar *str2)
{
   return strcmp (str1->str, str2);
}

static gint
g_string_compare (const GString *str1, const GString *str2)
{
   return strcmp (str1->str, str2->str);
}

/*----------------------------------------------------------------------------
 * Project Edit Dialog Utility Functions
 *--------------------------------------------------------------------------*/
/*
 * As people are added or deleted from the list of project members, we need
 * to add or remove their names from the auto-assign widget.  The next two
 * functions handle that logic.
 */
#ifdef USE_NEW_GTK
static void 
add_to_auto_assign (GtkComboBox  *combo,
                    const gchar  *name,
                    const gchar  *id)
{
   GList       *names;
   GList       *ids;
   GList       *iter;
   gint         idx;
   GString     *namestr;
   GString     *idstr;

   ids = g_object_get_data (G_OBJECT (combo), ID_LIST);
   names = g_object_get_data (G_OBJECT (combo), NAME_LIST);

   namestr = g_string_new (name);
   idstr = g_string_new (id);

   idx = 0;
   for (iter = g_list_first (names);
        iter != NULL;
        iter = iter->next)
      {
      /*
       * If the current name is the NULL_NAME, skip it.  That one
       * should always be first.
       */
      if (g_string_strcmp ((GString *)iter->data, NULL_NAME) == 0)
         {
         idx++;
         continue;
         }

      if (g_string_compare ((GString *)iter->data, namestr) >= 0)
         {
         break;
         }
      idx++;
      }

   names = g_list_insert (names, namestr, idx);
   ids = g_list_insert (ids, idstr, idx);

   gtk_combo_box_insert_text (combo, idx, name);

   g_object_set_data (G_OBJECT (combo), ID_LIST, ids);
   g_object_set_data (G_OBJECT (combo), NAME_LIST, names);
}


static void 
remove_from_auto_assign (GtkComboBox *combo,
                         const gchar *id)
{
   GList     *names;
   GList     *ids;
   GList     *lst_ptr;
   GList     *sel_data;
   gint       idx;
   gint       sel;

   names = g_object_get_data (G_OBJECT (combo), NAME_LIST);
   g_assert (names != NULL);
   ids = g_object_get_data (G_OBJECT (combo), ID_LIST);
   g_assert (ids != NULL);

   /*
    * Determine which item is currently selected. 
    */
   sel = gtk_combo_box_get_active (combo);
   sel_data = g_list_nth (names, sel);

   /*
    * Find the id in the id list, remove the link.  Remove the name
    * at the same position in the name list.
    */
   lst_ptr = g_list_find_custom (ids, id, (GCompareFunc)g_string_strcmp);
   idx = g_list_position (ids, lst_ptr);
   g_assert (idx != -1);

   ids = g_list_remove_link (ids, lst_ptr);
   g_string_free ((GString *)lst_ptr->data, TRUE);
   g_list_free (lst_ptr);

   lst_ptr = g_list_nth (names, idx);
   names = g_list_remove_link (names, lst_ptr);
   g_string_free ((GString *)lst_ptr->data, TRUE);
   g_list_free (lst_ptr);

   gtk_combo_box_remove_text (combo, idx);

   /*
    * If we are removing the currently selected item, select
    * the "no name" item.
    */
   if (sel == idx)
      {
      gtk_combo_box_set_active (combo, 0);
      }

   g_object_set_data (G_OBJECT (combo), ID_LIST, ids);
   g_object_set_data (G_OBJECT (combo), NAME_LIST, names);
}
#else
static void 
add_to_auto_assign (GtkOptionMenu *opt_menu,
                    const gchar   *name,
                    const gchar   *id)
{
   GList       *names;
   GList       *ids;
   GList       *iter;
   GtkWidget   *menu; 
   GtkWidget   *item;
   gint         idx;
   GString     *namestr;
   GString     *idstr;

   menu = gtk_option_menu_get_menu (opt_menu);
   if (menu == NULL)
      {
      menu = gtk_menu_new ();
      gtk_option_menu_set_menu (opt_menu, menu);
      }

   ids = g_object_get_data (G_OBJECT (opt_menu), ID_LIST);
   names = g_object_get_data (G_OBJECT (opt_menu), NAME_LIST);

   namestr = g_string_new (name);
   idstr = g_string_new (id);

   idx = 0;
   for (iter = g_list_first (names);
         iter != NULL;
         iter = iter->next)
      {
      /*
       * If the current name is the NULL_NAME, skip it.  That one
       * should always be first.
       */
      if (g_string_strcmp ((GString *)iter->data, NULL_NAME) == 0)
         {
         idx++;
         continue;
         }

      if (g_string_compare ((GString *)iter->data, namestr) >= 0)
         {
         break;
         }
      idx++;
      }

   names = g_list_insert (names, namestr, idx);
   ids = g_list_insert (ids, idstr, idx);

   item = gtk_menu_item_new_with_label (name);
   gtk_menu_shell_insert (GTK_MENU_SHELL (menu), item, idx);
   gtk_widget_show (item);

   g_object_set_data (G_OBJECT (opt_menu), ID_LIST, ids);
   g_object_set_data (G_OBJECT (opt_menu), NAME_LIST, names);
}


static void 
remove_from_auto_assign (GtkOptionMenu *opt_menu,
                         const gchar   *id)
{
   GList     *names;
   GList     *ids;
   GList     *lst_ptr;
   GList     *sel_data;
   GtkWidget *menu;
   GtkWidget *item;
   gint       idx;
   gint       sel;

   names = g_object_get_data (G_OBJECT (opt_menu), NAME_LIST);
   g_assert (names != NULL);
   ids = g_object_get_data (G_OBJECT (opt_menu), ID_LIST);
   g_assert (ids != NULL);

   /*
    * Save off the current selection, so we can restore it after the
    * menu is re-built (assuming it was not deleted).
    */
   sel = gtk_option_menu_get_history (opt_menu);
   sel_data = g_list_nth (names, sel);

   /*
    * Find the id in the id list, remove the link.  Remove the name
    * at the same position in the name list.
    */
   lst_ptr = g_list_find_custom (ids, id, (GCompareFunc)g_string_strcmp);
   idx = g_list_position (ids, lst_ptr);
   g_assert (idx != -1);

   ids = g_list_remove_link (ids, lst_ptr);
   g_string_free ((GString *)lst_ptr->data, TRUE);
   g_list_free (lst_ptr);

   lst_ptr = g_list_nth (names, idx);
   names = g_list_remove_link (names, lst_ptr);
   g_string_free ((GString *)lst_ptr->data, TRUE);
   g_list_free (lst_ptr);

   /*
    * Evidently, we cannot remove items from a menu, so just use the 
    * names to rebuild the array.
    */
   gtk_option_menu_remove_menu (opt_menu);
   menu = gtk_menu_new ();
   sel = 0;
   for (lst_ptr = g_list_first (names);
         lst_ptr != NULL;
         lst_ptr = lst_ptr->next)
      {
      /*
       * If the current data is the saved off selection, we will need to
       * select it when we are done.
       */
      if (sel_data->data == lst_ptr->data)
         {
         sel = g_list_position (names, lst_ptr);
         }
      item = gtk_menu_item_new_with_label (((GString *)lst_ptr->data)->str);
      gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
      gtk_widget_show (item);
      }

   gtk_option_menu_set_menu (opt_menu, menu);
   gtk_option_menu_set_history (opt_menu, sel);

   g_object_set_data (G_OBJECT (opt_menu), ID_LIST, ids);
   g_object_set_data (G_OBJECT (opt_menu), NAME_LIST, names);
}
#endif


static gboolean 
rm_submitter (PGconn       *conn, 
              gint          project_num,
              const gchar  *login_id)
{
   GString  *buffer;
   PGresult *res;
   gboolean  ok;
   
   buffer = g_string_new ("");
   g_string_printf (buffer, RM_SUBMITTER_SQL, project_num, login_id);
   res = PQexec (conn, buffer->str);
   ok = chk_sql_error (res, _("Removing Submitters"));
   
   PQclear (res);
   g_string_free (buffer, TRUE);

   return ok;
}


static gboolean 
rm_responsible (PGconn       *conn, 
                gint          project_num,
                const gchar  *login_id)
{
   GString  *buffer;
   PGresult *res;
   gboolean  ok;
   
   buffer = g_string_new ("");
   g_string_printf (buffer, RM_RESPONSIBLE_SQL, project_num, login_id);
   res = PQexec (conn, buffer->str);
   ok = chk_sql_error (res, _("Removing Responsible"));
   
   PQclear (res);
   g_string_free (buffer, TRUE);

   return ok;
}


static gboolean 
insert_submitter (PGconn      *conn, 
                  gint         project_num,
                  const gchar *login_id)
{
   GString  *buffer;
   PGresult *res;
   gboolean  ok;

   START_TRANSACTION (conn);
   
   buffer = g_string_new ("");
   g_string_printf (buffer, INS_SUBMITTER_SQL,
                    project_num, login_id, login_id, project_num);
   res = PQexec (conn, buffer->str);
   ok = chk_sql_error (res, _("Inserting Submitter"));
   
   finalize_transaction (conn, res);
   PQclear (res);
   g_string_free (buffer, TRUE);

   return ok;
}


static gboolean 
insert_responsible (PGconn      *conn, 
                    gint         project_num,
                    const gchar *login_id)
{
   GString  *buffer;
   PGresult *res;
   gboolean  ok;

   START_TRANSACTION (conn);
   
   buffer = g_string_new ("");
   g_string_printf (buffer, INS_RESPONSIBLE_SQL, project_num, login_id);
   res = PQexec (conn, buffer->str);
   ok = chk_sql_error (res, _("Inserting Project Member"));
   
   finalize_transaction (conn, res);
   PQclear (res);
   g_string_free (buffer, TRUE);

   return ok;
}


/*
 * Queries the database for persons
 */
static PGresult *
read_person_data (PGconn *conn, gboolean show_inactive)
{
   GString  *buffer;
   PGresult *res;
   
   buffer = g_string_new (PERSON_QUERY);
   if (!show_inactive)
      {
      buffer = g_string_append (buffer, WHERE_PERSON_ACTIVE);
      }
   buffer = g_string_append (buffer, PERSON_ORDER_BY);

   res = PQexec (conn, buffer->str);
   
   g_string_free (buffer, TRUE);
   return res;
}


/*
 * get_next_version_pk
 *
 * Get the next version number from the database for this project.  This 
 * is done at insertion time.  Up until then, the version number used 
 * in the version list is just a unique guess.
 */
static gint
get_next_version_pk (PGconn *conn, gint prj)
{
   GString   *sql_buffer;
   PGresult  *res;
   gint       ver_num = 0;

   sql_buffer = g_string_new ("");
   g_string_printf (sql_buffer, "SELECT next_version_num( %d )", prj);
   res = PQexec (conn, sql_buffer->str);
   if (chk_sql_error (res, _("Get next project version number")))
      {
      ver_num = atoi (PQgetvalue (res, 0, 0));
      }

   PQclear (res);
   g_string_free (sql_buffer, TRUE);

   return ver_num;
}


/*
 * can_edit_project
 *
 * Determine if the current user is able to edit the project.
 */
static gboolean
can_edit_project (PGconn *conn, gint pk)
{
   PGresult  *res;
   GString   *sql_buffer;
   gboolean   access = FALSE;
 
   sql_buffer = g_string_new ("");
   g_string_printf (sql_buffer, "SELECT can_edit_project (%d)", pk);

   res = PQexec (conn, sql_buffer->str);
   if (chk_sql_error (res, _("Getting project access")))
      {
      access = (toupper (PQgetvalue (res, 0, 0)[0]) == 'T');
      }
   g_string_free (sql_buffer, TRUE);
   PQclear (res);

   return access;
}


static gboolean
is_submitter (PGconn *conn, const gchar *id, gint prj)
{
   GString  *sql;
   GString  *user = g_string_new (id);
   PGresult *res;
   gboolean  rtn = FALSE;

   g_string_prepare_db_instr (user);
   sql = g_string_new ("SELECT 'x' FROM submitter ");
   g_string_append_printf (sql,
                           "WHERE login_id = %s AND project_num = %d",
                           user->str, prj);
   res = PQexec (conn, sql->str);
   if (chk_sql_error (res, _("Getting Submitter")))
      {
      rtn = (PQntuples (res) == 1);
      }

   PQclear (res);
   g_string_free (user, TRUE);
   g_string_free (sql, TRUE);

   return rtn;
}


static gboolean
is_prj_member (PGconn *conn, const gchar *id, gint prj)
{
   GString  *sql;
   GString  *user = g_string_new (id);
   PGresult *res;
   gboolean  rtn = FALSE;

   g_string_prepare_db_instr (user);
   sql = g_string_new ("SELECT 'x' FROM responsible ");
   g_string_append_printf (sql,
                           "WHERE login_id = %s AND project_num = %d",
                           user->str, prj);
   res = PQexec (conn, sql->str);
   if (chk_sql_error (res, _("Getting Submitter")))
      {
      rtn = (PQntuples (res) == 1);
      }

   PQclear (res);
   g_string_free (user, TRUE);
   g_string_free (sql, TRUE);

   return rtn;
}


/*
 * draw_name_tree
 *
 * Draw the name tree.  If combo is not NULL, add project members to it
 * also...
 */
#ifdef USE_NEW_GTK
static gboolean
draw_name_tree (GtkTreeView   *tree, 
                GtkComboBox   *combo,
                PGconn        *conn,
                gint           prj_pk,
                gboolean       show_inactive)
#else
static gboolean
draw_name_tree (GtkTreeView   *tree, 
                GtkOptionMenu *combo,
                PGconn        *conn,
                gint           prj_pk,
                gboolean       show_inactive)
#endif
{
   PGresult      *res;
   GtkListStore  *store;
   GtkTreeIter    iter;
   GString       *name_str;
   gboolean       active;
   gint           i;
   gint           n;
   gboolean       rtn_val = FALSE;
   gboolean       member;
   gchar         *id;

   res = read_person_data (conn, show_inactive);

   if (chk_sql_error (res, _("Reading Project Person Data")))
      {
      store = GTK_LIST_STORE (gtk_tree_view_get_model (tree));
      gtk_list_store_clear (store);
      n = PQntuples (res);
      name_str = g_string_new ("");
      for (i = 0; i < n; i++)
         {
         g_string_printf (name_str, "%s, %s",
                          PQgetvalue (res, i, LAST_NAME_POS),
                          PQgetvalue (res, i, FIRST_NAME_POS));
         active = (toupper (PQgetvalue (res, i, ACTIVE_POS)[0]) == 'T');
         id = PQgetvalue (res, i, LOGIN_ID_POS);
         member = is_prj_member (conn, id, prj_pk);
         gtk_list_store_append (store, &iter);
         gtk_list_store_set (store, &iter,
                             PERSON_NAME,
                             name_str->str,
                             PERSON_LOGIN_ID,
                             id,
                             PERSON_SUB,
                             is_submitter (conn, id, prj_pk),
                             PERSON_RESP,
                             member,
                             -1);
         if (member && combo != NULL)
            {
            GString  *str = g_string_new ("");

            g_string_printf (str, AUTO_ASSIGN_NAME_FORMAT, name_str->str, id);
            add_to_auto_assign (combo, str->str, id);
            g_string_free (str, TRUE);
            }
         }
      rtn_val = TRUE;
      g_string_free (name_str, TRUE);
      }
   PQclear (res);

   return rtn_val;
}


static void
draw_version_list (GtkTreeView *tree,
                   PGconn      *conn,
                   gint         prj_num,
                   gint         def_open_ver,
                   gint         def_close_ver,
                   gboolean     show_inactive)
{
   GtkListStore  *store;
   GtkTreeIter    iter;
   PGresult      *res;
   GString       *sql;
   gint           i, n;
   gboolean       is_active;
   gint           ver_num;

   store = GTK_LIST_STORE (gtk_tree_view_get_model (tree));

   sql = g_string_new ("");
   g_string_printf (sql, VERSION_QUERY, prj_num);
   if (!show_inactive)
      {
      sql = g_string_append (sql, WHERE_VERSION_ACTIVE);
      }
   sql = g_string_append (sql, VERSION_ORDER_BY);
   res = PQexec (conn, sql->str);
   if (chk_sql_error (res, _("Getting Version Information")))
      {
      gtk_list_store_clear (store);
      n = PQntuples (res);
      for (i = 0; i < n; i++)
         {
         is_active = (toupper ((PQgetvalue (res, i, ACTIVE_POS))[0]) == 'T');
         ver_num = atoi (PQgetvalue (res, i, PK_NUM_POS));
         gtk_list_store_append (store, &iter);
         gtk_list_store_set (store, &iter,
                             VERSION_NAME, PQgetvalue (res, i, NAME_POS),
                             VERSION_DESCR, PQgetvalue (res, i, DESCR_POS),
                             VERSION_ACTIVE, is_active,
                             VERSION_DEF_OPEN, (ver_num == def_open_ver),
                             VERSION_DEF_CLOSE, (ver_num == def_close_ver),
                             VERSION_NUM, ver_num,
                             -1);
         }
      }

   g_string_free (sql, TRUE);
   PQclear (res);
}


#ifdef USE_NEW_GTK
static void
initialize_leader_combo (GtkComboBox *combo,
                         PGconn      *conn,
                         gint         prj_num,
                         const gchar *ldr)
{
   GString   *sql;
   PGresult  *res;
   gint       i, n;
   GList     *ids = NULL;
   GString   *name;
   GString   *id;
   gint       sel;

   sql = g_string_new ("");
   g_string_printf (sql, LEADER_QUERY, prj_num);
   sql = g_string_append (sql, PERSON_ORDER_BY);

   res = PQexec (conn, sql->str);
   if (chk_sql_error (res, _("Fetch potential leaders")))
      {
      n = PQntuples (res);
      sel = 0;
      name = g_string_new ("");
      for (i = 0; i < n; i++)
         {
         /*
          * The id strings are to be destroyed when the dialog box is
          * destroyed (see on_project_edit_dlg_destroy() for details).
          */
         id = g_string_new (PQgetvalue (res, i, LOGIN_ID_POS));
         if (strcmp (ldr, id->str) == 0)
            {
            sel = i;
            }
         g_string_printf (name, "%s, %s (%s)",
                          PQgetvalue (res, i, LAST_NAME_POS),
                          PQgetvalue (res, i, FIRST_NAME_POS),
                          id->str);
         gtk_combo_box_append_text (combo, name->str);
         ids = g_list_append (ids, id);
         }
      g_object_set_data (G_OBJECT (combo), ID_LIST, ids);
      gtk_combo_box_set_active (combo, sel);
      g_string_free (name, TRUE);
      }

   PQclear (res);
   g_string_free (sql, TRUE);
}
#else
static void
initialize_leader_combo (GtkOptionMenu *opt,
                         PGconn        *conn,
                         gint           prj_num,
                         const gchar   *ldr)
{
   GString   *sql;
   PGresult  *res;
   gint       i, n;
   GList     *ids = NULL;
   GtkWidget *menu;
   GtkWidget *item;
   GString   *name;
   GString   *id;
   gint       sel;

   sql = g_string_new ("");
   g_string_printf (sql, LEADER_QUERY, prj_num);
   sql = g_string_append (sql, PERSON_ORDER_BY);

   res = PQexec (conn, sql->str);
   if (chk_sql_error (res, _("Fetch potential leaders")))
      {
      n = PQntuples (res);
      menu = gtk_menu_new ();
      sel = 0;
      name = g_string_new ("");
      for (i = 0; i < n; i++)
         {
         /*
          * The id strings are to be destroyed when the dialog box is
          * destroyed (see on_project_edit_dlg_destroy() for details).
          */
         id = g_string_new (PQgetvalue (res, i, LOGIN_ID_POS));
         if (strcmp (ldr, id->str) == 0)
            {
            sel = i;
            }
         g_string_printf (name, "%s, %s (%s)",
                          PQgetvalue (res, i, LAST_NAME_POS),
                          PQgetvalue (res, i, FIRST_NAME_POS),
                          id->str);
         item = gtk_menu_item_new_with_label (name->str);
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
         gtk_widget_show (item);
         ids = g_list_append (ids, id);
         }
      g_object_set_data (G_OBJECT (opt), ID_LIST, ids);
      gtk_option_menu_set_menu (opt, menu);
      gtk_option_menu_set_history (opt, sel);
      g_string_free (name, TRUE);
      }

   PQclear (res);
   g_string_free (sql, TRUE);
}
#endif


static void
update_prj_leader (GtkWidget *widget,
                   gpointer   user_data)
{
   PGconn   *conn;
   GString  *sql_buffer;
   PGresult *res;
   GList    *id_list;
   GList    *item;
   GString  *id;
   gint      sel;
   gint      pk_num;

   conn = g_object_get_data (G_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   
   pk_num = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (user_data),
                             PK_NUM)); 
   id_list = g_object_get_data (G_OBJECT (widget), ID_LIST);
   g_assert (id_list != NULL);

#ifdef USE_NEW_GTK
   sel = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
#else
   sel = gtk_option_menu_get_history (GTK_OPTION_MENU (widget));
#endif
   if (sel >= 0)
      {
      item = g_list_nth (id_list, sel);
      id = (GString *)item->data;

      sql_buffer = g_string_new ("");
      g_string_printf (sql_buffer, UPDATE_PRJ_LEADER, id->str, pk_num);

      START_TRANSACTION (conn);
      res = PQexec (conn, sql_buffer->str);
      chk_sql_error (res, _("Updating project leader"));
      finalize_transaction (conn, res);
      PQclear (res);
      g_string_free (sql_buffer, TRUE);
      }
}


static void
update_auto_assign (GtkWidget *widget,
                    gpointer   user_data)
{
   PGconn   *conn;
   GString  *sql_buffer;
   PGresult *res;
   GList    *id_list;
   GList    *item;
   GString  *id;
   gint      sel;
   gint      pk_num;

   conn = g_object_get_data (G_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   
   pk_num = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (user_data),
                             PK_NUM)); 
   g_assert (pk_num > 0);

   id_list = g_object_get_data (G_OBJECT (widget), ID_LIST);
   g_assert (id_list != NULL);

#ifdef USE_NEW_GTK
   sel = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
#else
   sel = gtk_option_menu_get_history (GTK_OPTION_MENU (widget));
#endif
   if (sel >= 0)
      {
      item = g_list_nth (id_list, sel);
      id = (GString *)item->data;
                           
      sql_buffer = g_string_new ("");
      if (g_string_strcmp (id, INVALID_ID) == 0)
         {
         g_string_printf (sql_buffer, UPDATE_AUTO_ASSIGN_NULL, pk_num);
         }
      else
         {
         g_string_printf (sql_buffer, UPDATE_AUTO_ASSIGN, id->str, pk_num);
         }

      START_TRANSACTION (conn);
      res = PQexec (conn, sql_buffer->str);
      chk_sql_error (res, _("Updating auto assign login id"));
      finalize_transaction (conn, res);
      PQclear (res);
      g_string_free (sql_buffer, TRUE);
      }
}


static gboolean
validate_project (GtkWidget  *dlg)
{
   GtkEntry         *prj_name_entry;
   GtkTreeView      *user_list_view;
   gboolean          retval = TRUE;
   GtkWidget        *msg_dlg;

   prj_name_entry = GTK_ENTRY (lookup_widget (dlg, NAME_ENTRY));
   g_assert (prj_name_entry);
   user_list_view = GTK_TREE_VIEW (lookup_widget (dlg, USER_LIST_VIEW));
   g_assert (user_list_view != NULL);

   /*
    * TODO:
    *
    * We should do some sort of a WARNING check on the submitters
    * and responsibles....
    */
   /*
    * For each check, only check if retval is still good (TRUE).
    */
   if (retval && strlen (gtk_entry_get_text (prj_name_entry)) == 0)
      {
      msg_dlg = gtk_message_dialog_new
         (GTK_WINDOW (dlg), 0,
          GTK_MESSAGE_ERROR,
          GTK_BUTTONS_OK,
          _("You must supply a project name."));
      gtk_dialog_run (GTK_DIALOG (msg_dlg));
      gtk_widget_destroy (msg_dlg);
      retval = FALSE;
      }

   return retval;
}


static GString *
build_prj_update_str (GtkDialog *dlg, gint pk_num)
{
   GtkEntry        *entry;
   GtkTextView     *text;
   GtkTextBuffer   *buff;
   GtkTextIter      start;
   GtkTextIter      end;
   GString         *upd;
   GString         *tmp;
   gchar           *char_buf;
   gboolean         first = TRUE;

   upd = g_string_new ("UPDATE project SET ");
   tmp = g_string_new ("");

   entry = GTK_ENTRY (lookup_widget (GTK_WIDGET (dlg), NAME_ENTRY));
   g_assert (entry != NULL);
   if (object_is_dirty (G_OBJECT (entry)))
      {
      tmp = g_string_assign (tmp, gtk_entry_get_text (entry));
      g_string_prepare_db_instr (tmp);
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "NAME = ");
      upd = g_string_append (upd, tmp->str);
      }

   text = GTK_TEXT_VIEW (lookup_widget (GTK_WIDGET (dlg), DESCR_TEXT));
   g_assert (text != NULL);
   buff = gtk_text_view_get_buffer (text);
   if (gtk_text_buffer_get_modified (buff))
      {
      gtk_text_buffer_get_bounds (buff, &start, &end);
      char_buf = gtk_text_buffer_get_text (buff, &start, &end, FALSE);
      tmp = g_string_new (char_buf);
      g_free (char_buf);
      g_string_prepare_db_instr (tmp);
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "DESCR = ");
      upd = g_string_append (upd, tmp->str);
      }

   upd = g_string_append (upd, " WHERE PROJECT_NUM = ");
   g_string_printf (tmp, "%d", pk_num);
   upd = g_string_append (upd, tmp->str);

   g_string_free (tmp, TRUE);

   return upd;
}


static gboolean 
apply_db_changes (GtkDialog *dlg)
{
   PGresult            *res;
   gint                 pk_num;
   gboolean             in_error = FALSE;
   PGconn              *conn;
   GString             *sqlstr;
   GtkTextView         *text;
   GtkTextBuffer       *buff;

   conn = g_object_get_data (G_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn != NULL);
   text = GTK_TEXT_VIEW (lookup_widget (GTK_WIDGET (dlg), DESCR_TEXT));
   g_assert (text != NULL);
   buff = gtk_text_view_get_buffer (text);

   pk_num = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dlg), PK_NUM)); 
   if (objects_children_are_dirty (G_OBJECT (dlg)) ||
         gtk_text_buffer_get_modified (buff))
      {
      START_TRANSACTION (conn);
      sqlstr = build_prj_update_str (dlg, pk_num);
      res = PQexec (conn, sqlstr->str);
      in_error = !chk_sql_error (res, _("updating project data"));
      g_string_free (sqlstr, TRUE);
      finalize_transaction (conn, res);
      PQclear (res);
      }
   
   if (!in_error)
      {
      set_all_clean (G_OBJECT (dlg));
      gtk_text_buffer_set_modified (buff, FALSE);
      }

   return !in_error;
}


/*----------------------------------------------------------------------------
 * Project Edit Signal Handlers
 *--------------------------------------------------------------------------*/
/*
 * Several toggle buttons are tied directly to column in the project table.
 * Use this function (and the enum above) to handle their toggles.
 *
 * The enumeration for the toggle button must be stored as object data of
 * the toggle button w/ key CHECK_BOX_ITEM.
 */
static void 
on_button_toggled (GtkToggleButton  *togglebutton,
                   gpointer          user_data)
{
   PGconn   *conn;
   PGresult *res;
   gint      cb_type;
   GString  *sql_buffer;
   gint      pk_num;

   conn = g_object_get_data (G_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   pk_num = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (user_data),
                             PK_NUM));
   g_assert (pk_num > 0);

   cb_type = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (togglebutton),
                                                 CHECK_BOX_ITEM));
   
   sql_buffer = g_string_new ("update project set ");
   switch (cb_type)
      {
      case NEW_NOTIFY_GRP:
         sql_buffer = g_string_append (sql_buffer, "new_pr_notify_grp = ");
         break;

      case NEW_NOTIFY_PL:
         sql_buffer = g_string_append (sql_buffer, "new_pr_notify_pl = ");
         break;

      case ASSIGN_NOTIFY_RESP:
         sql_buffer = g_string_append (sql_buffer, "assign_notify_resp = ");
         break;

      case ASSIGN_NOTIFY_SUB:
         sql_buffer = g_string_append (sql_buffer, "assign_notify_sub = ");
         break;

      case ASSIGN_NOTIFY_PL:
         sql_buffer = g_string_append (sql_buffer, "assign_notify_pl = ");
         break;

      case UPD_NOTIFY_GRP:
         sql_buffer = g_string_append (sql_buffer, "upd_notify_grp = ");
         break;

      case UPD_NOTIFY_PL:
         sql_buffer = g_string_append (sql_buffer, "upd_notify_pl = ");
         break;

      case UPD_NOTIFY_RESP:
         sql_buffer = g_string_append (sql_buffer, "upd_notify_resp = ");
         break;

      case UPD_NOTIFY_SUB:
         sql_buffer = g_string_append (sql_buffer, "upd_notify_sub = ");
         break;

      case PRJ_ACTIVE:
         sql_buffer = g_string_append (sql_buffer, "active = ");
         break;

      default:
         g_assert_not_reached ();
         break;
      }

   if (gtk_toggle_button_get_active (togglebutton))
      {
      sql_buffer = g_string_append (sql_buffer, "TRUE ");
      }
   else
      {
      sql_buffer = g_string_append (sql_buffer, "FALSE ");
      }

   g_string_append_printf (sql_buffer, "where project_num = %d ", pk_num);

   START_TRANSACTION (conn);
   res = PQexec (conn, sql_buffer->str);
   chk_sql_error (res, _("Updating project toggle value"));
   finalize_transaction (conn, res);
   g_string_free (sql_buffer, TRUE);
   PQclear (res);
}


/*
 * on_project_edit_dlg_show
 *
 * When the dialog is shown, several widgets will need to be initialized.
 * Also, at this time it is safe to attach the "changed" handlers to 
 * several of the widgets.
 */
static void 
on_project_edit_dlg_show (GtkWidget  *widget,
                          gpointer    user_data)
{
   gboolean         in_error = TRUE; /* set false if resp and submitter
                                        lists all draw OK. */
   PGconn          *conn;
   GString         *query;
   GtkWidget       *child;
   GtkTreeView     *tree;
   PGresult        *result;
   GtkToggleButton *show_inactive_cb;
   GList           *id_list = NULL;
   GList           *list_iter = NULL;
   gint             def_open_ver = INVALID_PK;
   gint             def_close_ver = INVALID_PK;
   gboolean         show_inactive;
   gint             pk_num;
   gint             sel;
   gint             idx;
   gchar           *id;
   GtkTextBuffer   *buff;
   GString         *title_str = g_string_new (_("Project"));


   conn = g_object_get_data (G_OBJECT (widget), MY_DB_CONN);
   g_assert (conn != NULL);
   pk_num = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), PK_NUM));

   tree = GTK_TREE_VIEW (lookup_widget (widget, USER_LIST_VIEW));
   g_assert (tree != NULL);
   show_inactive_cb = GTK_TOGGLE_BUTTON (lookup_widget (widget, 
                                                        SHOW_INACTIVE_USERS));
   g_assert (show_inactive_cb != NULL);
   show_inactive = gtk_toggle_button_get_active (show_inactive_cb);

   child = lookup_widget (widget, AUTO_ASSIGN_COMBO);
   g_assert (child != NULL);

#ifdef USE_NEW_GTK
   add_to_auto_assign (GTK_COMBO_BOX (child), NULL_NAME, INVALID_ID);
   if (draw_name_tree (tree, GTK_COMBO_BOX (child),
                       conn, pk_num, show_inactive))
      {
      in_error = FALSE;
      }
#else
   add_to_auto_assign (GTK_OPTION_MENU (child), NULL_NAME, INVALID_ID);
   if (draw_name_tree (tree, GTK_OPTION_MENU (child),
                       conn, pk_num, show_inactive))
      {
      in_error = FALSE;
      }
#endif
   
   if (!in_error)
      {
      query = g_string_new ("");
      g_string_printf (query, PROJECT_QUERY, pk_num);
 
      result = PQexec (conn, query->str);
      if (chk_sql_error (result, _("getting project data to edit")))
         {
         /*
          * At this point, the auto assign menu should be fully
          * initialized.  Intialize the value.
          */
         id_list = g_object_get_data (G_OBJECT (child), ID_LIST);
         g_assert (id_list != NULL);
         sel = 0;
         if (!PQgetisnull (result, 0, AUTO_ASSIGN_POS))
            {
            id = PQgetvalue (result, 0, AUTO_ASSIGN_POS);
            idx = 0;
            for (list_iter = g_list_first (id_list);
                  list_iter != NULL;
                  list_iter = list_iter->next)
               {
               if (g_string_strcmp ((GString *)list_iter->data, id) == 0)
                  {
                  sel = idx;
                  break;
                  }
               idx++;
               }
            }
#ifdef USE_NEW_GTK
         gtk_combo_box_set_active (GTK_COMBO_BOX (child), sel);
#else
         gtk_option_menu_set_history (GTK_OPTION_MENU (child), sel);
#endif
         g_signal_connect (child, "changed",
                           G_CALLBACK (update_auto_assign), widget);

         child = lookup_widget (widget, NAME_ENTRY);
         g_assert (child != NULL);
         gtk_entry_set_text (GTK_ENTRY (child),
               PQgetvalue (result, 0, NAME_POS));
         g_string_append_printf (title_str, " - %s", 
                                 PQgetvalue (result, 0, NAME_POS));
      
         child = lookup_widget (widget, DESCR_TEXT);
         g_assert (child != NULL);
         buff = gtk_text_view_get_buffer (GTK_TEXT_VIEW (child));
         gtk_text_buffer_set_text (buff,
                                   PQgetvalue (result, 0, DESCR_POS), -1);
         gtk_text_buffer_set_modified (buff, FALSE);
      
         child = lookup_widget (widget, LEADER_COMBO);
         g_assert (child != NULL);
#ifdef USE_NEW_GTK
         initialize_leader_combo (GTK_COMBO_BOX (child), conn, pk_num,
                                  PQgetvalue (result, 0, LOGIN_ID_POS));
#else
         initialize_leader_combo (GTK_OPTION_MENU (child), conn, pk_num,
                                  PQgetvalue (result, 0, LOGIN_ID_POS));
#endif
         g_signal_connect (child, "changed",
                           G_CALLBACK (update_prj_leader), widget);

         child = lookup_widget (widget, ACTIVE_CB);
         g_assert (child != NULL);
         gtk_toggle_button_set_active 
            (GTK_TOGGLE_BUTTON (child),
             toupper ((PQgetvalue (result, 0, ACTIVE_POS))[0]) =='T');
         g_signal_connect (child, "toggled",
                           G_CALLBACK (on_button_toggled), widget);

         child = lookup_widget (widget, NEW_NOTIFY_GRP_CB); 
         g_assert (child != NULL);
         gtk_toggle_button_set_active 
            (GTK_TOGGLE_BUTTON (child),
             toupper ((PQgetvalue (result, 0, NEW_PR_GRP_POS))[0]) =='T');
         g_signal_connect (child, "toggled",
                           G_CALLBACK (on_button_toggled), widget);

         child = lookup_widget (widget, NEW_NOTIFY_PL_CB); 
         g_assert (child != NULL);
         gtk_toggle_button_set_active 
            (GTK_TOGGLE_BUTTON (child),
             toupper ((PQgetvalue (result, 0, NEW_PR_PL_POS))[0]) =='T');
         g_signal_connect (child, "toggled",
                           G_CALLBACK (on_button_toggled), widget);

         child = lookup_widget (widget, ASSIGN_NOTIFY_RESP_CB); 
         g_assert (child != NULL);
         gtk_toggle_button_set_active 
            (GTK_TOGGLE_BUTTON (child),
             toupper ((PQgetvalue (result, 0, ASSIGN_RESP_POS))[0]) =='T');
         g_signal_connect (child, "toggled",
                           G_CALLBACK (on_button_toggled), widget);

         child = lookup_widget (widget, ASSIGN_NOTIFY_SUB_CB); 
         g_assert (child != NULL);
         gtk_toggle_button_set_active 
            (GTK_TOGGLE_BUTTON (child),
             toupper ((PQgetvalue (result, 0, ASSIGN_SUB_POS))[0]) =='T');
         g_signal_connect (child, "toggled",
                           G_CALLBACK (on_button_toggled), widget);

         child = lookup_widget (widget, ASSIGN_NOTIFY_PL_CB); 
         g_assert (child != NULL);
         gtk_toggle_button_set_active 
            (GTK_TOGGLE_BUTTON (child),
             toupper ((PQgetvalue (result, 0, ASSIGN_PL_POS))[0]) =='T');
         g_signal_connect (child, "toggled",
                           G_CALLBACK (on_button_toggled), widget);

         child = lookup_widget (widget, UPD_NOTIFY_GRP_CB); 
         g_assert (child != NULL);
         gtk_toggle_button_set_active 
            (GTK_TOGGLE_BUTTON (child),
             toupper ((PQgetvalue (result, 0, UPD_GRP_POS))[0]) =='T');
         g_signal_connect (child, "toggled",
                           G_CALLBACK (on_button_toggled), widget);

         child = lookup_widget (widget, UPD_NOTIFY_PL_CB); 
         g_assert (child != NULL);
         gtk_toggle_button_set_active 
            (GTK_TOGGLE_BUTTON (child),
             toupper ((PQgetvalue (result, 0, UPD_PL_POS))[0]) =='T');
         g_signal_connect (child, "toggled",
                           G_CALLBACK (on_button_toggled), widget);

         child = lookup_widget (widget, UPD_NOTIFY_RESP_CB); 
         g_assert (child != NULL);
         gtk_toggle_button_set_active 
            (GTK_TOGGLE_BUTTON (child),
             toupper ((PQgetvalue (result, 0, UPD_RESP_POS))[0]) =='T');
         g_signal_connect (child, "toggled",
                           G_CALLBACK (on_button_toggled), widget);

         child = lookup_widget (widget, UPD_NOTIFY_SUB_CB); 
         g_assert (child != NULL);
         gtk_toggle_button_set_active 
            (GTK_TOGGLE_BUTTON (child),
             toupper ((PQgetvalue (result, 0, UPD_SUB_POS))[0]) =='T');
         g_signal_connect (child, "toggled",
                           G_CALLBACK (on_button_toggled), widget);

         if (PQgetisnull (result, 0, OPEN_VER_POS))
            {
            def_open_ver = INVALID_PK;
            }
         else
            {
            def_open_ver = atoi (PQgetvalue (result, 0, OPEN_VER_POS));
            }

         if (PQgetisnull (result, 0, CLOSE_VER_POS))
            {
            def_close_ver = INVALID_PK;
            }
         else
            {
            def_close_ver = atoi (PQgetvalue (result, 0, CLOSE_VER_POS));
            }
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (result);
      g_string_free (query, TRUE);

      /*
       * If there has not been a error to this point, fetch the version
       * information for the current project and display it.
       */
      if (!in_error)
         {
         child = lookup_widget (widget, SHOW_ALL_VER_CB);
         g_assert (child != NULL);
         show_inactive = gtk_toggle_button_get_active
            (GTK_TOGGLE_BUTTON (child));
         child = lookup_widget (widget, VER_LIST_VIEW);
         g_assert (child != NULL);
         draw_version_list (GTK_TREE_VIEW (child), conn,
                            pk_num, def_open_ver, def_close_ver,
                            show_inactive);
         }
      }
   
   if (in_error)
      {
      gtk_widget_destroy (widget);
      }
   else
      {
      set_all_clean (G_OBJECT (widget));
      gtk_window_set_title (GTK_WINDOW (widget), title_str->str);
      }

   g_string_free (title_str, TRUE);
}


static void 
on_project_edit_dlg_destroy (GObject  *object,
                             gpointer  user_data)
{
   GList         *temp_list;
   GList         *list_iter;
   GtkWidget     *widget;
   GString       *str;

   /*
    * Destroy the ID List associated with the leader combo.
    */
   widget = lookup_widget (GTK_WIDGET (object), LEADER_COMBO);
   g_assert (widget != NULL);

   temp_list = g_object_get_data (G_OBJECT (widget), ID_LIST);
   g_assert (temp_list != NULL);
   for (list_iter = g_list_first (temp_list);
         list_iter != NULL;
         list_iter = list_iter->next)
      {
      str = (GString *)list_iter->data;
      g_string_free (str, TRUE);
      }
   g_list_free (temp_list);

   /*
    * Destroy the ID List associated with the auto assign combo.
    *
    * For sorting, the auto assign combo also has a name list that
    * needs to be destroyed.
    */
   widget = lookup_widget (GTK_WIDGET (object), AUTO_ASSIGN_COMBO);
   temp_list = g_object_get_data (G_OBJECT (widget), NAME_LIST);
   g_assert (temp_list != NULL);
   for (list_iter = g_list_first (temp_list);
         list_iter != NULL;
         list_iter = list_iter->next)
      {
      str = (GString *)list_iter->data;
      g_string_free (str, TRUE);
      }
   g_list_free (temp_list);
   temp_list = g_object_get_data (G_OBJECT (widget), ID_LIST);
   g_assert (temp_list != NULL);
   for (list_iter = g_list_first (temp_list);
         list_iter != NULL;
         list_iter = list_iter->next)
      {
      str = (GString *)list_iter->data;
      g_string_free (str, TRUE);
      }
   g_list_free (temp_list);

   /*
    * Destroy the "dirty" list...
    */
   destroy_dirty_list (object);
}


static void 
on_show_inactive_ver_cb_toggled (GtkToggleButton  *togglebutton,
                                 gpointer          user_data)
{
   gboolean      show_inactive;
   GtkTreeView  *tree;
   gint          def_open_ver = INVALID_PK;
   gint          def_close_ver = INVALID_PK;
   gint          prj_pk;
   GString      *sql;
   PGconn       *conn;
   PGresult     *res;

   tree = GTK_TREE_VIEW (lookup_widget (GTK_WIDGET (user_data),
                                        VER_LIST_VIEW));
   g_assert (tree != NULL);
   conn = g_object_get_data (G_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   prj_pk = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (user_data), PK_NUM));
   g_assert (prj_pk > 0);

   sql = g_string_new ("select open_version, close_version from project ");
   g_string_append_printf (sql, "where project_num = %d ", prj_pk);

   res = PQexec (conn, sql->str);
   if (chk_sql_error (res, _("Get default open and close versions")))
      {
      if (PQgetisnull (res, 0, 0))
         {
         def_open_ver = INVALID_PK;
         }
      else
         {
         def_open_ver = atoi (PQgetvalue (res, 0, 0));
         }
      if (PQgetisnull (res, 0, 1))
         {
         def_close_ver = INVALID_PK;
         }
      else
         {
         def_close_ver = atoi (PQgetvalue (res, 0, 1));
         }

      show_inactive = gtk_toggle_button_get_active (togglebutton);
      draw_version_list (tree, conn, prj_pk, def_open_ver, def_close_ver,
                         show_inactive);
      }
   PQclear (res);
   g_string_free (sql, TRUE);
}


static void
on_show_inactive_users_toggled (GtkToggleButton *togglebutton,
                                gpointer         data)
{
   GtkTreeView *tree;
   PGconn      *conn;
   gint         prj_pk;
   gboolean     show_inactive;

   conn = g_object_get_data (G_OBJECT (data), MY_DB_CONN);
   g_assert (conn != NULL);
   tree = GTK_TREE_VIEW (lookup_widget (GTK_WIDGET (data), USER_LIST_VIEW));
   g_assert (tree != NULL);
   prj_pk = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (data), PK_NUM));

   show_inactive = gtk_toggle_button_get_active (togglebutton);

   draw_name_tree (tree, NULL, conn, prj_pk, show_inactive);
}


static void
on_close_button_clicked (GtkDialog *dlg)
{
   GtkWidget  *parent;
   GtkWidget  *main_win;
   PGconn     *conn;

   conn = g_object_get_data (G_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn != NULL);
   parent = g_object_get_data (G_OBJECT (dlg), MY_PARENT);
   g_assert (parent != NULL);
   /*
    * The main win should be the parent of the project list dialog, 
    * which should be the parent of this edit dialog.
    */
   main_win = lookup_widget (parent, MY_PARENT);
   g_assert (main_win != NULL);

   if (validate_project (GTK_WIDGET (dlg)))
      {
      if (apply_db_changes (dlg))
         {
         /*
          * refresh the project list dialog (draw_project_list()), AND
          * on the main window (refresh_project_list()).
          */
         draw_project_list (GTK_WINDOW (parent), conn);
         refresh_project_list (main_win);
         gtk_widget_destroy (GTK_WIDGET (dlg));
         }
      }
}


static void
on_project_edit_dlg_clicked (GtkDialog *dlg,
                             gint       btn_cd,
                             gpointer   user_data)
{
   GConfClient  *client;

   client = g_object_get_data (G_OBJECT (dlg), MY_GCONF_CLIENT);
   g_assert (client != NULL);
   save_dlg_size (client, GTK_WIDGET (dlg), PRJ_EDIT_DLG_SEC);

   switch (btn_cd)
      {
      case GTK_RESPONSE_CLOSE:
         on_close_button_clicked (dlg);
         break;
      case GTK_RESPONSE_DELETE_EVENT:
         break;
      default:
         g_assert_not_reached();
      }
}



static void
user_toggled (GtkCellRendererToggle *cell,
              gchar                 *path_str,
              gpointer               data)
{
#ifdef USE_NEW_GTK
   GtkComboBox   *combo;
#else
   GtkOptionMenu *combo;
#endif
   GtkTreeView   *tree;
   GtkTreePath   *path = gtk_tree_path_new_from_string (path_str);
   GtkListStore  *store;
   GtkTreeIter    iter;
   PGconn        *conn;
   gint           column;
   gint           prj_num;
   gchar         *id;
   gchar         *name;
   gboolean       sub;
   gboolean       resp;
   GString       *name_str;

#ifdef USE_NEW_GTK
   combo = GTK_COMBO_BOX (lookup_widget (GTK_WIDGET (data),
                          AUTO_ASSIGN_COMBO));
#else
   combo = GTK_OPTION_MENU (lookup_widget (GTK_WIDGET (data),
                            AUTO_ASSIGN_COMBO));
#endif
   g_assert (combo != NULL);
   tree = GTK_TREE_VIEW (lookup_widget (GTK_WIDGET (data), USER_LIST_VIEW));
   g_assert (tree != NULL);
   store = GTK_LIST_STORE (gtk_tree_view_get_model (tree));
   conn = g_object_get_data (G_OBJECT (data), MY_DB_CONN);
   g_assert (conn != NULL);
   prj_num = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (data), PK_NUM));
   column = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell), COLUMN_NUM));
   g_assert (column == PERSON_SUB || column == PERSON_RESP);

   gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
   gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
                       PERSON_NAME, &name,
                       PERSON_LOGIN_ID, &id,
                       PERSON_SUB, &sub,
                       PERSON_RESP, &resp,
                       -1);

   switch (column)
      {
      case PERSON_SUB:
         if (sub)
            {
            sub = FALSE;
            rm_submitter (conn, prj_num, id);
            }
         else
            {
            sub = TRUE;
            insert_submitter (conn, prj_num, id);
            }
         break;
 
      case PERSON_RESP:
         if (resp)
            {
            resp = FALSE;
            rm_responsible (conn, prj_num, id);
            remove_from_auto_assign (combo, id);
            }
         else
            {
            resp = TRUE;
            insert_responsible (conn, prj_num, id);
            name_str = g_string_new ("");
            g_string_printf (name_str, AUTO_ASSIGN_NAME_FORMAT, name, id);
            add_to_auto_assign (combo, name_str->str, id);
            g_string_free (name_str, TRUE);
            }
         break;

      default:
         g_assert_not_reached();
         break;
      }
   gtk_list_store_set (store, &iter, PERSON_SUB, sub, PERSON_RESP, resp, -1);

   g_free (id);
   g_free (name);
   gtk_tree_path_free (path);
}


static void
version_edited (GtkCellRendererText *cell,
                const gchar         *path_str,
                const gchar         *new_text,
                gpointer             data)
{
   GtkTreeView   *tree;
   GtkListStore  *store;
   gint           column;
   GtkTreePath   *path = gtk_tree_path_new_from_string (path_str);
   GtkTreeIter    iter;
   gint           prj_pk;
   gint           ver_num;
   PGconn        *conn;
   PGresult      *res;
   GString       *sql;
   GString       *txt;

   txt = g_string_new (new_text);
   g_string_prepare_db_instr (txt);
   if (strcmp (txt->str, "NULL") != 0)
      {
      conn = g_object_get_data (G_OBJECT (data), MY_DB_CONN);
      g_assert (conn != NULL);
      prj_pk = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (data), PK_NUM));
      g_assert (prj_pk > 0);
      column = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell),
                                                   COLUMN_NUM));
      g_assert (column == VERSION_NAME || column == VERSION_DESCR);
      tree = GTK_TREE_VIEW (lookup_widget (GTK_WIDGET (data), VER_LIST_VIEW));
      g_assert (tree != NULL);
      store = GTK_LIST_STORE (gtk_tree_view_get_model (tree));
      gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
      gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
                          VERSION_NUM, &ver_num, -1);

      sql = g_string_new ("update project_version ");
      switch (column)
         {
         case VERSION_NAME:
            g_string_append_printf (sql, "set version_text = %s ", txt->str);
            break;
         case VERSION_DESCR:
            g_string_append_printf (sql, "set descr = %s ", txt->str);
            break;
         default:
            g_assert_not_reached ();
            break;
         }
      g_string_append_printf (sql, "where project_num = %d ", prj_pk);
      g_string_append_printf (sql, "and version_num = %d ", ver_num);

      START_TRANSACTION (conn);

      res = PQexec (conn, sql->str);
      if (chk_sql_error (res, _("Update version descr/name")))
         {
         gtk_list_store_set (store, &iter, column, new_text, -1);
         }
      finalize_transaction (conn, res);
      PQclear (res);
      g_string_free (sql, TRUE);
      }
   gtk_tree_path_free (path);
   g_string_free (txt, TRUE);
}


static void
version_toggled (GtkCellRendererToggle *cell,
                 gchar                 *path_str,
                 gpointer               data)
{
   GtkTreeView  *tree;
   GtkTreePath  *path = gtk_tree_path_new_from_string (path_str);
   GtkTreeModel *model;
   GtkTreeIter   iter;
   gboolean      active;
   gint          cnum;
   gint          ver_num;
   PGconn       *conn;
   PGresult     *res;
   gint          prj_pk;
   GString      *sql;

   tree = GTK_TREE_VIEW (lookup_widget (GTK_WIDGET (data), VER_LIST_VIEW));
   g_assert (tree != NULL);
   conn = g_object_get_data (G_OBJECT (data), MY_DB_CONN);
   g_assert (conn != NULL);
   prj_pk = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (data), PK_NUM));
   g_assert (prj_pk > 0);

   cnum = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell), COLUMN_NUM));
   sql = g_string_new ("");
   
   /*
    * Get the important data from the selected line
    */
   model = gtk_tree_view_get_model (tree);
   gtk_tree_model_get_iter (model, &iter, path);
   gtk_tree_model_get (model, &iter,
                       VERSION_ACTIVE, &active,
                       VERSION_NUM, &ver_num,
                       -1);

   switch (cnum)
      {
      case VERSION_ACTIVE:
         active = !active;
         sql = g_string_assign (sql, "update project_version ");
         if (active)
            {
            sql = g_string_append (sql, "set active = TRUE ");
            }
         else
            {
            sql = g_string_append (sql, "set active = FALSE ");
            }
         g_string_append_printf (sql, "where project_num = %d ", prj_pk);
         g_string_append_printf (sql, "and version_num = %d ", ver_num);
         break;

      case VERSION_DEF_OPEN:
      case VERSION_DEF_CLOSE:
         active = TRUE;
         gtk_tree_model_get_iter_first (model, &iter);
         do
            {
            gtk_list_store_set (GTK_LIST_STORE (model), &iter,
                                cnum, !active, -1);
            } while (gtk_tree_model_iter_next (model, &iter));
         gtk_tree_model_get_iter (model, &iter, path);

         sql = g_string_assign (sql, "update project ");
         if (cnum == VERSION_DEF_OPEN)
            {
            g_string_append_printf (sql, "set open_version = %d ", ver_num);
            }
         else
            {
            g_string_append_printf (sql, "set close_version = %d ", ver_num);
            }
         g_string_append_printf (sql, "where project_num = %d ", prj_pk);
         break;

      default:
         g_assert_not_reached();
         break;
      }

   START_TRANSACTION (conn);

   res = PQexec (conn, sql->str);
   if (chk_sql_error (res, _("Updating version status")))
      {
      gtk_list_store_set (GTK_LIST_STORE (model), &iter,
                          cnum, active,
                          -1);
      }

   finalize_transaction (conn, res);
   PQclear (res);
   g_string_free (sql, TRUE);
   gtk_tree_path_free (path);
}


static void
on_ver_rm (GtkWidget *widget, gpointer data)
{
   GtkTreeSelection  *sel;
   GtkTreeView       *tree;
   GtkListStore      *store;
   GtkTreeIter        iter;
   PGconn            *conn;
   PGresult          *res;
   gint               prj_num;
   gint               ver_num;
   GString           *sql;
   GtkWidget         *msg_dlg;

   tree = GTK_TREE_VIEW (lookup_widget (GTK_WIDGET (data), VER_LIST_VIEW));
   g_assert (tree != NULL);
   conn = g_object_get_data (G_OBJECT (data), MY_DB_CONN);
   g_assert (conn != NULL);
   prj_num = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (data), PK_NUM));
   g_assert (prj_num > 0);

   store = GTK_LIST_STORE (gtk_tree_view_get_model (tree));
   sel = gtk_tree_view_get_selection (tree);

   if (gtk_tree_selection_get_selected (sel, NULL, &iter))
      {
      msg_dlg = gtk_message_dialog_new
         (GTK_WINDOW (data),
          GTK_DIALOG_DESTROY_WITH_PARENT,
          GTK_MESSAGE_QUESTION,
          GTK_BUTTONS_YES_NO,
          _("Deleting a versoin cannot be reversed.  Are you sure you want to continue?"));
      if (gtk_dialog_run (GTK_DIALOG (msg_dlg)) == GTK_RESPONSE_YES)
         {
         gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
                             VERSION_NUM, &ver_num, -1);
         START_TRANSACTION (conn);
         sql = g_string_new ("delete from project_version ");
         g_string_append_printf (sql, "where project_num = %d ", prj_num);
         g_string_append_printf (sql, "and version_num = %d ", ver_num);
         res = PQexec (conn, sql->str);
         if (chk_sql_error (res, _("Deleting Version")))
            {
            gtk_list_store_remove (store, &iter);
            }
         g_string_free (sql, TRUE);
         finalize_transaction (conn, res);
         PQclear (res);
         }
      gtk_widget_destroy (msg_dlg);
      }
   else
      {
      msg_dlg = gtk_message_dialog_new
         (GTK_WINDOW (data),
          GTK_DIALOG_DESTROY_WITH_PARENT,
          GTK_MESSAGE_INFO,
          GTK_BUTTONS_OK,
          _("To delete a version, you must select a version to delete."));
      gtk_dialog_run (GTK_DIALOG (msg_dlg));
      gtk_widget_destroy (msg_dlg);
      }
}


static void
on_ver_add (GtkWidget *widget, gpointer data)
{
   GtkTreeView   *tree;
   GtkListStore  *store;
   GtkTreeIter    iter;
   PGconn        *conn;
   PGresult      *res;
   gint           prj_num;
   gint           ver_num;
   GString       *name = g_string_new (_("v"));
   GString       *descr = g_string_new (_("Describe Version Here"));
   GString       *name_display;
   GString       *descr_display;
   GString       *sql;

   tree = GTK_TREE_VIEW (lookup_widget (GTK_WIDGET (data), VER_LIST_VIEW));
   g_assert (tree != NULL);
   conn = g_object_get_data (G_OBJECT (data), MY_DB_CONN);
   g_assert (conn != NULL);
   prj_num = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (data), PK_NUM));
   g_assert (prj_num > 0);

   ver_num = get_next_version_pk (conn, prj_num);
   g_string_append_printf (name, "%d", ver_num);
   name_display = g_string_new (name->str);
   descr_display = g_string_new (descr->str);
   g_string_prepare_db_instr (name);
   g_string_prepare_db_instr (descr);

   START_TRANSACTION (conn);

   sql =  g_string_new ("");
   g_string_printf (sql, INSERT_VERSION, ver_num, prj_num, name->str,
                    descr->str, "TRUE", ver_num);
   res = PQexec (conn, sql->str);
   if (chk_sql_error (res, _("Insert New Version")))
      {
      store = GTK_LIST_STORE (gtk_tree_view_get_model (tree));
      gtk_list_store_append (store, &iter);
      gtk_list_store_set (store, &iter,
                          VERSION_NAME, name_display->str,
                          VERSION_DESCR, descr_display->str,
                          VERSION_ACTIVE, TRUE,
                          VERSION_DEF_OPEN, FALSE,
                          VERSION_DEF_CLOSE, FALSE,
                          VERSION_NUM, ver_num,
                          -1);
      }

   finalize_transaction (conn, res);
   PQclear (res);
   g_string_free (name, TRUE);
   g_string_free (descr, TRUE);
   g_string_free (name_display, TRUE);
   g_string_free (descr_display, TRUE);
   g_string_free (sql, TRUE);
}


/*----------------------------------------------------------------------------
 * Project Edit Dialog Main Functions
 *--------------------------------------------------------------------------*/
/*
 * create_project_edit_dlg
 *
 * Create the project edit dialog.  Note that for many of the widgets, the
 * "changed" signals are not connected until after they are shown.  This is
 * done in on_project_edit_dlg_show().
 */
static GtkWidget* 
create_project_edit_dlg (gint       pk_num,
                         PGconn    *conn,
                         GtkWidget *parent,
                         gboolean   user_is_admin)
{
  GtkWidget           *project_edit_dlg;
  GtkWidget           *dialog_vbox14;
  GtkWidget           *notebook2;
  GtkWidget           *prj_tab_vbox;
  GtkSizeGroup        *size_group;
  GtkWidget           *name_entry;
  GtkWidget           *descr_entry;
  GtkWidget           *auto_combo;
  GtkWidget           *leader_combo;
  GtkWidget           *active_cb;
  GtkWidget           *scrolledwindow1;
  GtkWidget           *scrolledwindow6;
  GtkWidget           *user_vbox;
  GtkWidget           *user_list_view;
  GtkListStore        *submitter_list_store;
  GtkWidget           *label;
  GtkWidget           *info_lbl;
  GtkWidget           *submitter_label;
  GtkWidget           *notify_vbox;
  GtkWidget           *hbox35;
  GtkWidget           *new_pr_notify_grp_cb;
  GtkWidget           *new_pr_notify_pl_cb;
  GtkWidget           *assign_pr_notify_resp_cb;
  GtkWidget           *assign_pr_notify_sub_cb;
  GtkWidget           *assign_pr_notify_pl_cb;
  GtkWidget           *upd_pr_notify_grp_cb;
  GtkWidget           *upd_pr_notify_resp_cb;
  GtkWidget           *upd_pr_notify_sub_cb;
  GtkWidget           *upd_pr_notify_pl_cb;
  GtkWidget           *version_lbl;
  GtkWidget           *scrolledwindow14;
  GtkListStore        *ver_list_store;
  GtkWidget           *ver_list_view;
  GtkWidget           *hseparator;
  GtkWidget           *hbox;
  GtkWidget           *ver_vbox;
  GtkWidget           *ver_add_btn;
  GtkWidget           *ver_del_btn;
  GtkWidget           *show_inactive_cb;
  GtkWidget           *warn_hbox;
  GtkWidget           *warn_icon;
  GtkWidget           *warn_lbl;
  GtkTooltips         *tooltips;
  gboolean             can_edit;
  GtkTextBuffer       *buff;
  GtkTreeViewColumn   *col;
  GtkCellRenderer     *rend;
  GtkWidget           *vbox_outer;
  GtkWidget           *hbox_inner;
  GtkWidget           *vbox_inner;
  GConfClient         *client;

  tooltips = g_object_get_data (G_OBJECT (parent), TOOLTIPS);
  g_assert (tooltips != NULL);
  client = g_object_get_data (G_OBJECT (parent), MY_GCONF_CLIENT);
  g_assert (client != NULL);

  project_edit_dlg = gtk_dialog_new_with_buttons
     (NULL, GTK_WINDOW (parent), GTK_DIALOG_DESTROY_WITH_PARENT,
      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
  GLADE_HOOKUP_OBJECT_NO_REF (project_edit_dlg, project_edit_dlg,
                              PROJECT_EDIT_DIALOG);
  gtk_container_set_border_width (GTK_CONTAINER (project_edit_dlg), 5);
  gtk_window_set_resizable (GTK_WINDOW (project_edit_dlg), TRUE);
  g_signal_connect (project_edit_dlg, "show",
                    G_CALLBACK (on_project_edit_dlg_show), NULL);
  g_signal_connect (project_edit_dlg, "destroy",
                    G_CALLBACK (on_project_edit_dlg_destroy), NULL);
  g_signal_connect (project_edit_dlg, "response",
                    G_CALLBACK (on_project_edit_dlg_clicked), NULL);
  set_dlg_size (client, project_edit_dlg, PRJ_EDIT_DLG_SEC, 500, 265);

  g_object_set_data (G_OBJECT (project_edit_dlg), TOP_DOG, project_edit_dlg);
  g_object_set_data (G_OBJECT (project_edit_dlg), MY_PARENT, parent);
  g_object_set_data (G_OBJECT (project_edit_dlg), MY_DB_CONN, conn);
  g_object_set_data (G_OBJECT (project_edit_dlg), MY_GCONF_CLIENT, client);
  g_object_set_data (G_OBJECT (project_edit_dlg), PK_NUM,
                     GINT_TO_POINTER (pk_num));
  gtk_dialog_set_default_response (GTK_DIALOG (project_edit_dlg),
                                   GTK_RESPONSE_CLOSE);
  can_edit = can_edit_project (conn, pk_num);

  dialog_vbox14 = GTK_DIALOG (project_edit_dlg)->vbox;
  GLADE_HOOKUP_OBJECT (project_edit_dlg, dialog_vbox14, "dialog_vbox14");
  gtk_box_set_spacing (GTK_BOX (dialog_vbox14), DTL_BOX_SPACING);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox14),
                                  STD_DLG_BORDER);
  gtk_widget_show (dialog_vbox14);

  notebook2 = gtk_notebook_new ();
  GLADE_HOOKUP_OBJECT (project_edit_dlg, notebook2, "notebook2");
  gtk_widget_show (notebook2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox14), notebook2, TRUE, TRUE, 0);

  /*
   * Start the Project Tab
   */
  prj_tab_vbox = gtk_vbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, prj_tab_vbox, "prj_tab_vbox");
  gtk_widget_show (prj_tab_vbox);
  gtk_container_add (GTK_CONTAINER (notebook2), prj_tab_vbox);
  gtk_container_set_border_width (GTK_CONTAINER (prj_tab_vbox),
                                  STD_DLG_BORDER / 2);

  size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);

  /*
   * Project Name
   */
  hbox = gtk_hbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox, "prj_name_hbox");
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (prj_tab_vbox), hbox, FALSE, FALSE, 0);

  label = gtk_label_new (_("Project:"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, label, "label1");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_size_group_add_widget (size_group, label);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  name_entry = gtk_entry_new ();
  GLADE_HOOKUP_OBJECT (project_edit_dlg, name_entry, NAME_ENTRY);
  gtk_widget_show (name_entry);
  gtk_box_pack_start (GTK_BOX (hbox), name_entry, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, name_entry, 
                        _("The name of the project (required)."), NULL);
  gtk_widget_set_sensitive (name_entry, can_edit);
  g_signal_connect (name_entry, "changed", G_CALLBACK (set_dirty), NULL);
  add_object_to_dirty_list (G_OBJECT (project_edit_dlg),
                            G_OBJECT (name_entry));

  /*
   * Description
   */
  hbox = gtk_hbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox, "prj_descr_hbox");
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (prj_tab_vbox), hbox, TRUE, TRUE, 0);

  label = gtk_label_new (_("Description:"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, label, "label2");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_size_group_add_widget (size_group, label);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, scrolledwindow1, "scrolledwindow1");
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (hbox), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1),
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow1),
                                       GTK_SHADOW_IN);

  descr_entry = gtk_text_view_new ();
  GLADE_HOOKUP_OBJECT (project_edit_dlg, descr_entry, DESCR_TEXT);
  gtk_widget_show (descr_entry);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), descr_entry);
  gtk_widget_set_size_request (descr_entry, 400, -1);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (descr_entry), TRUE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (descr_entry), GTK_WRAP_WORD);
  gtk_tooltips_set_tip (tooltips, 
                        descr_entry, 
                        _("A short description of the project (optional)."), 
                        NULL);
  gtk_widget_set_sensitive (descr_entry, can_edit);
  buff = gtk_text_view_get_buffer (GTK_TEXT_VIEW (descr_entry));

  /*
   * Project Leader
   */
  hbox = gtk_hbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox, "prj_ldr_hbox");
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (prj_tab_vbox), hbox, FALSE, FALSE, 0);

  label = gtk_label_new (_("Leader:"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, label, "label10");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_size_group_add_widget (size_group, label);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

#ifdef USE_NEW_GTK
  leader_combo = gtk_combo_box_new_text ();
#else
  leader_combo = gtk_option_menu_new ();
#endif
  GLADE_HOOKUP_OBJECT (project_edit_dlg, leader_combo, LEADER_COMBO);
  gtk_widget_set_sensitive (leader_combo, can_edit);
  gtk_box_pack_start (GTK_BOX (hbox), leader_combo, TRUE, TRUE, 0);
  gtk_widget_show (leader_combo);

  active_cb = gtk_check_button_new_with_label (_("Active"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, active_cb, ACTIVE_CB);
  gtk_widget_show (active_cb);
  gtk_box_pack_start (GTK_BOX (prj_tab_vbox), active_cb, FALSE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (active_cb), TRUE);
  gtk_widget_set_sensitive (active_cb, can_edit);
  g_object_set_data (G_OBJECT (active_cb), CHECK_BOX_ITEM,
                     GINT_TO_POINTER (PRJ_ACTIVE));

  info_lbl = gtk_label_new (_("Project Information"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, info_lbl, "info_lbl");
  gtk_widget_show (info_lbl);
  gtk_notebook_set_tab_label 
     (GTK_NOTEBOOK (notebook2),
      gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook2), 0),
      info_lbl);
  gtk_label_set_justify (GTK_LABEL (info_lbl), GTK_JUSTIFY_LEFT);


  /*
   * Start the Users tab
   */
  user_vbox = gtk_vbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, user_vbox, "user_vbox");
  gtk_widget_show (user_vbox);
  gtk_container_add (GTK_CONTAINER (notebook2), user_vbox);
  gtk_container_set_border_width (GTK_CONTAINER (user_vbox), 
                                  STD_DLG_BORDER / 2);

  scrolledwindow6 = gtk_scrolled_window_new (NULL, NULL);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, scrolledwindow6, "scrolledwindow6");
  gtk_widget_show (scrolledwindow6);
  gtk_box_pack_start (GTK_BOX (user_vbox), scrolledwindow6, 
                      TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow6), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow6),
                                       GTK_SHADOW_ETCHED_IN);

  submitter_list_store = gtk_list_store_new (NUM_PERSON_COLS,
                                             G_TYPE_STRING,
                                             G_TYPE_STRING,
                                             G_TYPE_BOOLEAN,
                                             G_TYPE_BOOLEAN,
                                             G_TYPE_BOOLEAN);
  user_list_view = gtk_tree_view_new_with_model
     (GTK_TREE_MODEL (submitter_list_store));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, user_list_view, USER_LIST_VIEW);
  gtk_container_add (GTK_CONTAINER (scrolledwindow6), user_list_view);
  gtk_widget_show (user_list_view);

  rend = gtk_cell_renderer_toggle_new ();
  g_object_set (rend, "activatable", can_edit, NULL);
  g_object_set_data (G_OBJECT (rend), COLUMN_NUM,
                     GINT_TO_POINTER (PERSON_SUB));
  g_signal_connect (rend, "toggled", G_CALLBACK (user_toggled),
                    project_edit_dlg);
  col = gtk_tree_view_column_new_with_attributes (_("Submitter"), rend,
                                                  "active", PERSON_SUB,
                                                  NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (user_list_view), col);

  rend = gtk_cell_renderer_toggle_new ();
  g_object_set (rend, "activatable", can_edit, NULL);
  g_object_set_data (G_OBJECT (rend), COLUMN_NUM,
                     GINT_TO_POINTER (PERSON_RESP));
  g_signal_connect (rend, "toggled", G_CALLBACK (user_toggled),
                    project_edit_dlg);
  col = gtk_tree_view_column_new_with_attributes (_("Member"), rend,
                                                  "active", PERSON_RESP,
                                                  NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (user_list_view), col);

  rend = gtk_cell_renderer_text_new ();
  col = gtk_tree_view_column_new_with_attributes (_("Login ID"), rend,
                                                  "text", PERSON_LOGIN_ID,
                                                  NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (user_list_view), col);

  rend = gtk_cell_renderer_text_new ();
  col = gtk_tree_view_column_new_with_attributes (_("Name"), rend,
                                                  "text", PERSON_NAME,
                                                  NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (user_list_view), col);


  show_inactive_cb = gtk_check_button_new_with_label 
     (_("Show Inactive Users"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, show_inactive_cb,
                       SHOW_INACTIVE_USERS);
  gtk_widget_show (show_inactive_cb);
  gtk_box_pack_start (GTK_BOX (user_vbox), show_inactive_cb,
                      FALSE, TRUE, 0);
  g_signal_connect (show_inactive_cb, "toggled",
                    G_CALLBACK (on_show_inactive_users_toggled),
                    project_edit_dlg);

  hbox = gtk_hbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox, "auto_assign_hbox");
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (user_vbox), hbox, FALSE, FALSE, 0);

  label = gtk_label_new (_("Auto-Assign new PRs to:"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, label, "aalbl");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

#ifdef USE_NEW_GTK
  auto_combo = gtk_combo_box_new_text ();
#else
  auto_combo = gtk_option_menu_new ();
#endif
  GLADE_HOOKUP_OBJECT (project_edit_dlg, auto_combo, AUTO_ASSIGN_COMBO);
  gtk_widget_set_sensitive (auto_combo, can_edit);
  gtk_box_pack_start (GTK_BOX (hbox), auto_combo, TRUE, TRUE, 0);
  gtk_widget_show (auto_combo);

  submitter_label = gtk_label_new (_("Users"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, submitter_label, "submitter_label");
  gtk_widget_show (submitter_label);
  gtk_notebook_set_tab_label 
     (GTK_NOTEBOOK (notebook2),
      gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook2), 1),
      submitter_label);
  gtk_label_set_justify (GTK_LABEL (submitter_label), GTK_JUSTIFY_LEFT);

  /*
   * Start the Notification Messages tab
   */
  notify_vbox = gtk_vbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, notify_vbox, "notify_vbox");
  gtk_widget_show (notify_vbox);
  gtk_container_add (GTK_CONTAINER (notebook2), notify_vbox);
  gtk_container_set_border_width (GTK_CONTAINER (notify_vbox), 5);

  hbox35 = gtk_hbox_new (FALSE, STD_BOX_SPACING * 3);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox35, "hbox35");
  gtk_widget_show (hbox35);
  gtk_box_pack_start (GTK_BOX (notify_vbox), hbox35, TRUE, TRUE, 0);

  /*
   * On New Problem Report...
   */
  vbox_outer = gtk_vbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox35, "newpr_out_vbox");
  gtk_widget_show (vbox_outer);
  gtk_box_pack_start (GTK_BOX (hbox35), vbox_outer, TRUE, TRUE, 0);

  label = gtk_label_new (NULL);
  gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\">New Problem Report</span>"));
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, label, "newpr_label");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox_outer), label, FALSE, FALSE, 0);

  hbox_inner = gtk_hbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox35, "newpr_in_hbox");
  gtk_widget_show (hbox_inner);
  gtk_box_pack_start (GTK_BOX (vbox_outer), hbox_inner, TRUE, TRUE, 0);

  label = gtk_label_new ("    ");
  GLADE_HOOKUP_OBJECT (project_edit_dlg, label, "newpr_space_label");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox_inner), label, FALSE, FALSE, 0);

  vbox_inner = gtk_vbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox35, "newpr_in_vbox");
  gtk_widget_show (vbox_inner);
  gtk_box_pack_start (GTK_BOX (hbox_inner), vbox_inner, TRUE, TRUE, 0);

  new_pr_notify_grp_cb = gtk_check_button_new_with_label (_("Notify Group"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, new_pr_notify_grp_cb,
                       NEW_NOTIFY_GRP_CB);
  gtk_widget_show (new_pr_notify_grp_cb);
  gtk_box_pack_start (GTK_BOX (vbox_inner), new_pr_notify_grp_cb,
                      FALSE, FALSE, 0);
  gtk_widget_set_sensitive (new_pr_notify_grp_cb, can_edit);
  g_object_set_data (G_OBJECT (new_pr_notify_grp_cb), CHECK_BOX_ITEM,
                     GINT_TO_POINTER (NEW_NOTIFY_GRP));

  new_pr_notify_pl_cb = gtk_check_button_new_with_label 
     (_("Notify Project Leader"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, new_pr_notify_pl_cb,
                       NEW_NOTIFY_PL_CB);
  gtk_widget_show (new_pr_notify_pl_cb);
  gtk_box_pack_start (GTK_BOX (vbox_inner), new_pr_notify_pl_cb,
                      FALSE, FALSE, 0);
  gtk_widget_set_sensitive (new_pr_notify_pl_cb, can_edit);
  g_object_set_data (G_OBJECT (new_pr_notify_pl_cb), CHECK_BOX_ITEM,
                     GINT_TO_POINTER (NEW_NOTIFY_PL));


  /*
   * On Assign Problem Report...
   */
  vbox_outer = gtk_vbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox35, "ass_out_vbox");
  gtk_widget_show (vbox_outer);
  gtk_box_pack_start (GTK_BOX (hbox35), vbox_outer, TRUE, TRUE, 0);

  label = gtk_label_new (NULL);
  gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\">Assign Problem Report</span>"));
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, label, "ass_label");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox_outer), label, FALSE, FALSE, 0);

  hbox_inner = gtk_hbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox35, "ass_in_hbox");
  gtk_widget_show (hbox_inner);
  gtk_box_pack_start (GTK_BOX (vbox_outer), hbox_inner, TRUE, TRUE, 0);

  label = gtk_label_new ("    ");
  GLADE_HOOKUP_OBJECT (project_edit_dlg, label, "ass_space_label");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox_inner), label, FALSE, FALSE, 0);

  vbox_inner = gtk_vbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox35, "ass_in_vbox");
  gtk_widget_show (vbox_inner);
  gtk_box_pack_start (GTK_BOX (hbox_inner), vbox_inner, TRUE, TRUE, 0);

  assign_pr_notify_resp_cb = 
     gtk_check_button_new_with_label (_("Notify Assignee"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, assign_pr_notify_resp_cb,
                       ASSIGN_NOTIFY_RESP_CB);
  gtk_widget_show (assign_pr_notify_resp_cb);
  gtk_box_pack_start (GTK_BOX (vbox_inner), 
                      assign_pr_notify_resp_cb, 
                      FALSE, FALSE, 0);
  gtk_widget_set_sensitive (assign_pr_notify_resp_cb, can_edit);
  g_object_set_data (G_OBJECT (assign_pr_notify_resp_cb), CHECK_BOX_ITEM,
                     GINT_TO_POINTER (ASSIGN_NOTIFY_RESP));

  assign_pr_notify_sub_cb = 
     gtk_check_button_new_with_label (_("Notify Submitter"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, assign_pr_notify_sub_cb,
                       ASSIGN_NOTIFY_SUB_CB);
  gtk_widget_show (assign_pr_notify_sub_cb);
  gtk_box_pack_start (GTK_BOX (vbox_inner), 
                      assign_pr_notify_sub_cb, 
                      FALSE, FALSE, 0);
  gtk_widget_set_sensitive (assign_pr_notify_sub_cb, can_edit);
  g_object_set_data (G_OBJECT (assign_pr_notify_sub_cb), CHECK_BOX_ITEM,
                     GINT_TO_POINTER (ASSIGN_NOTIFY_SUB));

  assign_pr_notify_pl_cb = 
     gtk_check_button_new_with_label (_("Notify Project Leader"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, assign_pr_notify_pl_cb,
                       ASSIGN_NOTIFY_PL_CB);
  gtk_widget_show (assign_pr_notify_pl_cb);
  gtk_box_pack_start (GTK_BOX (vbox_inner), 
                      assign_pr_notify_pl_cb, 
                      FALSE, FALSE, 0);
  gtk_widget_set_sensitive (assign_pr_notify_pl_cb, can_edit);
  g_object_set_data (G_OBJECT (assign_pr_notify_pl_cb), CHECK_BOX_ITEM,
                     GINT_TO_POINTER (ASSIGN_NOTIFY_PL));

  /*
   * On Update PR Status...
   */
  vbox_outer = gtk_vbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox35, "upstat_out_vbox");
  gtk_widget_show (vbox_outer);
  gtk_box_pack_start (GTK_BOX (hbox35), vbox_outer, TRUE, TRUE, 0);

  label = gtk_label_new (NULL);
  gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\">Update PR Status</span>"));
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, label, "upstat_label");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox_outer), label, FALSE, FALSE, 0);

  hbox_inner = gtk_hbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox35, "upstat_in_hbox");
  gtk_widget_show (hbox_inner);
  gtk_box_pack_start (GTK_BOX (vbox_outer), hbox_inner, TRUE, TRUE, 0);

  label = gtk_label_new ("    ");
  GLADE_HOOKUP_OBJECT (project_edit_dlg, label, "upstat_space_label");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox_inner), label, FALSE, FALSE, 0);

  vbox_inner = gtk_vbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox35, "upstat_in_vbox");
  gtk_widget_show (vbox_inner);
  gtk_box_pack_start (GTK_BOX (hbox_inner), vbox_inner, TRUE, TRUE, 0);

  upd_pr_notify_grp_cb = gtk_check_button_new_with_label (_("Notify Group"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, upd_pr_notify_grp_cb,
                       UPD_NOTIFY_GRP_CB);
  gtk_widget_show (upd_pr_notify_grp_cb);
  gtk_box_pack_start (GTK_BOX (vbox_inner), upd_pr_notify_grp_cb,
                      FALSE, FALSE, 0);
  gtk_widget_set_sensitive (upd_pr_notify_grp_cb, can_edit);
  g_object_set_data (G_OBJECT (upd_pr_notify_grp_cb), CHECK_BOX_ITEM,
                     GINT_TO_POINTER (UPD_NOTIFY_GRP));

  upd_pr_notify_resp_cb = gtk_check_button_new_with_label 
     (_("Notify Assignee"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, upd_pr_notify_resp_cb,
                       UPD_NOTIFY_RESP_CB);
  gtk_widget_show (upd_pr_notify_resp_cb);
  gtk_box_pack_start (GTK_BOX (vbox_inner), upd_pr_notify_resp_cb,
                      FALSE, FALSE, 0);
  gtk_widget_set_sensitive (upd_pr_notify_resp_cb, can_edit);
  g_object_set_data (G_OBJECT (upd_pr_notify_resp_cb), CHECK_BOX_ITEM,
                     GINT_TO_POINTER (UPD_NOTIFY_RESP));

  upd_pr_notify_sub_cb = gtk_check_button_new_with_label
     (_("Notify Submitter"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, upd_pr_notify_sub_cb,
                       UPD_NOTIFY_SUB_CB);
  gtk_widget_show (upd_pr_notify_sub_cb);
  gtk_box_pack_start (GTK_BOX (vbox_inner), upd_pr_notify_sub_cb,
                      FALSE, FALSE, 0);
  gtk_widget_set_sensitive (upd_pr_notify_sub_cb, can_edit);
  g_object_set_data (G_OBJECT (upd_pr_notify_sub_cb), CHECK_BOX_ITEM,
                     GINT_TO_POINTER (UPD_NOTIFY_SUB));

  upd_pr_notify_pl_cb = 
     gtk_check_button_new_with_label (_("Notify Project Leader"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, upd_pr_notify_pl_cb,
                       UPD_NOTIFY_PL_CB);
  gtk_widget_show (upd_pr_notify_pl_cb);
  gtk_box_pack_start (GTK_BOX (vbox_inner), upd_pr_notify_pl_cb,
                      FALSE, FALSE, 0);
  gtk_widget_set_sensitive (upd_pr_notify_pl_cb, can_edit);
  g_object_set_data (G_OBJECT (upd_pr_notify_pl_cb), CHECK_BOX_ITEM,
                     GINT_TO_POINTER (UPD_NOTIFY_PL));

  label = gtk_label_new (_("Notification Messages"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, label, "label33");
  gtk_widget_show (label);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook2), 
                              gtk_notebook_get_nth_page 
                                (GTK_NOTEBOOK (notebook2), 2), 
                              label);                                      

  /*
   * Start the Versions tab
   */
  ver_vbox = gtk_vbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, ver_vbox, "ver_vbox");
  gtk_widget_show (ver_vbox);
  gtk_container_add (GTK_CONTAINER (notebook2), ver_vbox);
  gtk_container_set_border_width (GTK_CONTAINER (ver_vbox), 5);

  scrolledwindow14 = gtk_scrolled_window_new (NULL, NULL);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, scrolledwindow14, "scrolledwindow14");
  gtk_widget_show (scrolledwindow14);
  gtk_box_pack_start (GTK_BOX (ver_vbox), scrolledwindow14, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow14), 
                                  GTK_POLICY_AUTOMATIC, 
                                  GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow14),
                                       GTK_SHADOW_ETCHED_IN);

  ver_list_store = gtk_list_store_new (NUM_VERSION_COLS,
                                       G_TYPE_STRING,
                                       G_TYPE_STRING,
                                       G_TYPE_BOOLEAN,
                                       G_TYPE_BOOLEAN,
                                       G_TYPE_BOOLEAN,
                                       G_TYPE_INT);
  ver_list_view = gtk_tree_view_new_with_model
     (GTK_TREE_MODEL (ver_list_store));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, ver_list_view, VER_LIST_VIEW);
  gtk_widget_show (ver_list_view);
  gtk_container_add (GTK_CONTAINER (scrolledwindow14), ver_list_view);

  rend = gtk_cell_renderer_toggle_new ();
  g_object_set (rend, "activatable", can_edit, NULL);
  g_object_set_data (G_OBJECT (rend), COLUMN_NUM,
                     GINT_TO_POINTER (VERSION_ACTIVE));
  g_signal_connect (rend, "toggled", G_CALLBACK (version_toggled),
                    project_edit_dlg);
  col = gtk_tree_view_column_new_with_attributes (_("Active"), rend,
                                                  "active", VERSION_ACTIVE,
                                                  NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (ver_list_view), col);

  rend = gtk_cell_renderer_toggle_new ();
  g_object_set (rend, "activatable", can_edit, NULL);
  g_object_set_data (G_OBJECT (rend), COLUMN_NUM,
                     GINT_TO_POINTER (VERSION_DEF_OPEN));
  g_signal_connect (rend, "toggled", G_CALLBACK (version_toggled),
                    project_edit_dlg);
  gtk_cell_renderer_toggle_set_radio (GTK_CELL_RENDERER_TOGGLE (rend), TRUE);
  col = gtk_tree_view_column_new_with_attributes (_("Def Open Ver"), rend,
                                                  "active",
                                                  VERSION_DEF_OPEN,
                                                  NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (ver_list_view), col);

  rend = gtk_cell_renderer_toggle_new ();
  g_object_set (rend, "activatable", can_edit, NULL);
  g_object_set_data (G_OBJECT (rend), COLUMN_NUM,
                     GINT_TO_POINTER (VERSION_DEF_CLOSE));
  g_signal_connect (rend, "toggled", G_CALLBACK (version_toggled),
                    project_edit_dlg);
  gtk_cell_renderer_toggle_set_radio (GTK_CELL_RENDERER_TOGGLE (rend), TRUE);
  col = gtk_tree_view_column_new_with_attributes (_("Def Close Ver"), rend,
                                                  "active",
                                                  VERSION_DEF_CLOSE,
                                                  NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (ver_list_view), col);

  rend = gtk_cell_renderer_text_new ();
  g_object_set (rend, "editable", can_edit, NULL);
  g_object_set_data (G_OBJECT (rend), COLUMN_NUM,
                     GINT_TO_POINTER (VERSION_NAME));
  g_signal_connect (rend, "edited", G_CALLBACK (version_edited),
                    project_edit_dlg);
  col = gtk_tree_view_column_new_with_attributes (_("Version"), rend,
                                                  "text", VERSION_NAME,
                                                  NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (ver_list_view), col);

  rend = gtk_cell_renderer_text_new ();
  g_object_set (rend, "editable", can_edit, NULL);
  g_object_set_data (G_OBJECT (rend), COLUMN_NUM,
                     GINT_TO_POINTER (VERSION_DESCR));
  g_signal_connect (rend, "edited", G_CALLBACK (version_edited),
                    project_edit_dlg);
  col = gtk_tree_view_column_new_with_attributes (_("Description"), rend,
                                                  "text", VERSION_DESCR,
                                                   NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (ver_list_view), col);

  hseparator = gtk_hseparator_new ();
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hseparator, "hsep2");
  gtk_widget_show (hseparator);
  gtk_box_pack_start (GTK_BOX (ver_vbox), hseparator, FALSE, FALSE, 0);

  hbox = gtk_hbox_new (TRUE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, hbox, "hbuttonbox1");
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (ver_vbox), hbox, FALSE, FALSE, 0);

  ver_add_btn = gtk_button_new_from_stock (GTK_STOCK_ADD);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, ver_add_btn, ADD_VER_BUTTON);
  gtk_widget_show (ver_add_btn);
  gtk_box_pack_start (GTK_BOX (hbox), ver_add_btn, TRUE, TRUE, 0);
  gtk_widget_set_sensitive (ver_add_btn, can_edit);
  g_signal_connect (ver_add_btn, "clicked",
                    G_CALLBACK (on_ver_add), project_edit_dlg);

  ver_del_btn = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
  GLADE_HOOKUP_OBJECT (project_edit_dlg, ver_del_btn, DEL_VER_BUTTON);
  gtk_widget_show (ver_del_btn);
  gtk_box_pack_start (GTK_BOX (hbox), ver_del_btn, TRUE, TRUE, 0);
  gtk_widget_set_sensitive (ver_del_btn, can_edit);
  g_signal_connect (ver_del_btn, "clicked",
                    G_CALLBACK (on_ver_rm), project_edit_dlg);

  show_inactive_cb = gtk_check_button_new_with_label 
     (_("Show Inactive Versions"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, show_inactive_cb, SHOW_ALL_VER_CB);
  gtk_widget_show (show_inactive_cb);
  gtk_box_pack_start (GTK_BOX (hbox), show_inactive_cb, TRUE, TRUE, 0);
  g_signal_connect (show_inactive_cb, "toggled",
                    G_CALLBACK (on_show_inactive_ver_cb_toggled),
                    project_edit_dlg);


  version_lbl = gtk_label_new (_("Versions"));
  GLADE_HOOKUP_OBJECT (project_edit_dlg, version_lbl, "version_lbl");
  gtk_widget_show (version_lbl);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook2), 
                              gtk_notebook_get_nth_page 
                                (GTK_NOTEBOOK (notebook2), 3), 
                              version_lbl);

  /*
   * If we cannot edit any of the projects, let the user know why editing
   * is disabled.
   */
  if (!can_edit)
     {
     warn_hbox = gtk_hbox_new (FALSE, STD_BOX_SPACING);
     GLADE_HOOKUP_OBJECT (project_edit_dlg, warn_hbox, "warn1_hbox");
     gtk_box_pack_start (GTK_BOX (dialog_vbox14), warn_hbox, FALSE, FALSE, 0);
     gtk_widget_show (warn_hbox);

     warn_icon = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING,
                                           GTK_ICON_SIZE_SMALL_TOOLBAR);
     GLADE_HOOKUP_OBJECT (project_edit_dlg, warn_icon, "warn1_icon");
     gtk_box_pack_start (GTK_BOX (warn_hbox), warn_icon, FALSE, FALSE, 0);
     gtk_widget_show (warn_icon);

     warn_lbl = gtk_label_new (NULL);
     GLADE_HOOKUP_OBJECT (project_edit_dlg, warn_lbl, "warn1_lbl");
     gtk_label_set_markup
        (GTK_LABEL (warn_lbl),
         _("<span weight=\"bold\">Editing Disabled:</span> Only the project leader and PRepS administrators can edit project data."));
     gtk_box_pack_start (GTK_BOX (warn_hbox), warn_lbl, FALSE, FALSE, 0);
     gtk_widget_show (warn_lbl);
     }

  g_object_set_data (G_OBJECT (project_edit_dlg), TOOLTIPS, tooltips);

  return project_edit_dlg;
}


static void
raise_or_create_project_edit_dlg (GtkWidget *parent,
                                  gint       pk_num,
                                  PGconn    *conn,
                                  gboolean   user_is_admin)
{
   GtkWidget  *dlg;
   GList      *child_list;
   GList      *list_iter;

   /*
    * Go through the existing list of children first.  If we have a
    * child that already uses this project's PK, just bring that one
    * to the top.
    */
   child_list = g_object_get_data (G_OBJECT (parent), my_child_list);

   list_iter = g_list_first (child_list);
   while (list_iter)
      {
      if (pk_num == GPOINTER_TO_INT
                      (g_object_get_data (G_OBJECT (list_iter->data), PK_NUM)))
         {
         gdk_window_show (GTK_WIDGET (list_iter->data)->window);
         gdk_window_raise (GTK_WIDGET (list_iter->data)->window);

         return;
         }
      list_iter = list_iter->next;
      }

   /*
    * Didn't find a child already displaying this project, so we need
    * to create one.
    */
   dlg = create_project_edit_dlg (pk_num, conn,
                                  parent, user_is_admin);
   gtk_widget_show (dlg);

   /*
    * Setup the child_dead() callback to remove the child from the 
    * child_list when the child dies.
    */
   g_signal_connect (dlg, "destroy", G_CALLBACK (child_dead),
                     (gpointer)my_child_list);

   child_list = g_list_append (child_list, dlg);
   g_object_set_data (G_OBJECT (parent), my_child_list, child_list);
}


/*---------------------------------------------------------------------------
 * Local support functions for the project list dialog box.
 *---------------------------------------------------------------------------
 */
static PGresult *
read_project_data (PGconn *conn, gboolean show_inactive)
{
   GString *buffer;
   PGresult *result;
   
   /*
    *  The order in the following SELECT is tied to #define's at
    *  the start of the file.  Change this SELECT, and be sure to
    *  change those to match.
    */
   buffer = g_string_new ("SELECT p.project_num, p.name, p.active, ");
   buffer = g_string_append (buffer, "p.login_id, p.descr, ");
   buffer = g_string_append (buffer, "u.last_name, u.first_name ");
   buffer = g_string_append (buffer, "FROM project p, person u ");
   buffer = g_string_append (buffer, "WHERE u.login_id = p.login_id "); 
   if (!show_inactive)
      {
      buffer = g_string_append (buffer, "AND p.active = TRUE ");
      }
   else
      {
      /*
       * This should not be needed, but for some reason, it is.
       */
      buffer = g_string_append 
         (buffer, "AND (p.active = TRUE OR p.active = FALSE) ");
      }
   buffer = g_string_append (buffer, "ORDER BY p.name ");
   
   result = PQexec (conn, buffer->str);
   
   g_string_free (buffer, TRUE);
   return result;
}


static void
delete_project (PGconn *conn, gint pk_num)
{
   GString  *buffer;
   PGresult *res;
   
   /*
    * If the open_version or close_version are not NULL, the delete from
    * project_version will fail.
    * 
    * This probably should be in the trigger.  I will open a problem report
    * to get this moved in there in a later version of the schema.  The 
    * DELETE from the project ideally would be the only thing in here.
    */
   START_TRANSACTION (conn);
   buffer = g_string_new
       ("UPDATE project SET open_version = NULL, close_version = NULL ");
   g_string_append_printf (buffer, "WHERE project_num = %d", pk_num);
   res = PQexec (conn, buffer->str);
   if (chk_sql_error (res, _("Update project open and close versions")))
      {
      PQclear (res);
      g_string_printf (buffer, "DELETE FROM project WHERE project_num = %d",
                       pk_num);
      res = PQexec (conn, buffer->str);
      chk_sql_error (res, _("Delete project"));
      }
   g_string_free (buffer, TRUE);
   finalize_transaction (conn, res);   
}


static gboolean 
draw_project_list (GtkWindow *parent, PGconn *conn)
{
   gint                ntuples;
   gint                i;
   gint                pk_num;
   PGresult           *res;
   GtkTreeView        *tree;
   GtkListStore       *store;
   GtkTreeIter         iter;
   GtkCheckButton     *cb_widget;
   gboolean            show_inactive;
   gboolean            rtn_val = FALSE;
   gboolean            is_admin;
   gboolean            active;
   GString            *name_str;
   
   tree = GTK_TREE_VIEW (lookup_widget (GTK_WIDGET (parent),
                                        PROJECT_LIST_VIEW));
   g_assert (tree != NULL);
   cb_widget = GTK_CHECK_BUTTON (lookup_widget (GTK_WIDGET (parent),
                                             SHOW_INACTIVE_CB)); 
   g_assert (cb_widget != NULL);
   show_inactive =
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_widget));
   is_admin = (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (parent),
                                                   ADMIN_USER)) == 1);
   
   res = read_project_data (conn, show_inactive);
   
   if (chk_sql_error (res, _("reading project data")))
      {
      store = GTK_LIST_STORE (gtk_tree_view_get_model (tree));
      gtk_list_store_clear (store);
      ntuples = PQntuples (res);
      name_str = g_string_new ("");
      
      for (i = 0; i < ntuples; i++)
         {
         char *descr;
         char *first_newline;

         /*
          * Chop the description off after the first new line char
          */
         descr = PQgetvalue (res, i, DESCR_POS);
         first_newline = strpbrk (descr, "\n");
         if (first_newline != NULL)
            {
            first_newline[0] = '\000';
            }
         
         pk_num = atoi (PQgetvalue (res, i, PK_NUM_POS));
         active = (toupper (PQgetvalue (res, i, ACTIVE_POS)[0]) == 'T');
         g_string_printf (name_str, "%s, %s", 
                          PQgetvalue (res, i, LAST_NAME_POS),
                          PQgetvalue (res, i, FIRST_NAME_POS));
         gtk_list_store_append (store, &iter);
         gtk_list_store_set (store, &iter,
                             PROJECT_NAME, PQgetvalue (res, i, NAME_POS),
                             PROJECT_DESCR, descr,
                             PROJECT_ACTIVE, active,
                             PROJECT_PK, pk_num,
                             PROJECT_EDITABLE, is_admin,
                             -1);
         }
      g_string_free (name_str, TRUE);
      rtn_val = TRUE;
      }
   
   PQclear (res);
   return rtn_val;
}



   
/*---------------------------------------------------------------------------
 * Signal handlers for the project list dialog box.
 *---------------------------------------------------------------------------
 */
static void 
on_project_dlg_show (GtkWidget  *widget, gpointer  user_data)
{
   PGconn          *conn;
   GtkToggleButton *cb;
   GString         *config_str;
   gboolean         flag;
   GConfClient     *client;
   GError          *error = NULL;

   config_str = g_string_new ("");

   client = g_object_get_data (G_OBJECT (widget), MY_GCONF_CLIENT);
   g_assert (client != NULL);

   cb = GTK_TOGGLE_BUTTON (lookup_widget (widget, SHOW_INACTIVE_CB));
   g_assert (cb != NULL);

   g_string_printf (config_str, "%s/%s/%s/%s", BASE_KEY, WINSTATES,
                    PRJ_DLG_SEC, SHOW_INACT);
   flag = gconf_client_get_bool (client, config_str->str, &error);
   if (error != NULL)
      {
      g_warning (error->message);
      g_error_free (error);
      error = NULL;
      flag = FALSE;
      }
   gtk_toggle_button_set_active (cb, flag);

   g_string_free (config_str, TRUE);

   conn = g_object_get_data (G_OBJECT (widget), MY_DB_CONN);
   g_assert (conn != NULL);
   if (!draw_project_list (GTK_WINDOW (widget), conn))
      {
      gtk_widget_destroy (widget);
      }
}


static void 
on_project_dlg_destroy (GObject  *object, gpointer  user_data)
{
   GList           *child_list;
   GList           *list_iter;
   GtkToggleButton *cb;
   GString         *config_str;
   GConfClient     *client;

   config_str = g_string_new ("");

   cb = GTK_TOGGLE_BUTTON (lookup_widget (GTK_WIDGET (object),
                           SHOW_INACTIVE_CB));
   g_assert (cb != NULL);
   client = g_object_get_data (object, MY_GCONF_CLIENT);
   g_assert (client != NULL);

   g_string_printf (config_str, "%s/%s/%s/%s", BASE_KEY, WINSTATES,
                    PRJ_DLG_SEC, SHOW_INACT);
   gconf_client_set_bool (client, config_str->str,
                          gtk_toggle_button_get_active (cb), NULL);

   g_string_free (config_str, TRUE);

   /*
    * Kill all of the children
    */
   child_list = g_object_get_data (object, my_child_list);
   list_iter = g_list_first (child_list);
   while (list_iter)
      {
      g_signal_handlers_block_by_func (list_iter->data,
                                       G_CALLBACK (child_dead),
                                       (gpointer)my_child_list);
      gtk_widget_destroy (GTK_WIDGET (list_iter->data));
      list_iter = list_iter->next;
      }
   g_list_free (child_list);
}


static void
on_project_dlg_clicked (GtkDialog *dlg, 
                        gint       btn,
                        gpointer   user_data)
{
   GConfClient  *client;

   client = g_object_get_data (G_OBJECT (dlg), MY_GCONF_CLIENT);
   g_assert (client != NULL);

   save_dlg_size (client, GTK_WIDGET (dlg), PRJ_DLG_SEC);
   gtk_widget_destroy (GTK_WIDGET (dlg));
}


static gint
create_new_project (PGconn *conn)
{
   gint       pk_num;
   PGresult  *res;
   GString   *str;

   START_TRANSACTION (conn);
   res = PQexec (conn, "select nextval ('project_seq')");
   g_assert (chk_sql_error (res, _("Get new project number")));
   pk_num = atoi (PQgetvalue (res, 0, 0));
   PQclear (res);

   str = g_string_new ("INSERT INTO project (project_num, name, active, ");
   str = g_string_append (str, "login_id) ");
   /*
    * The string used here for the new project name must be usable
    * in a SQL statement "as is"...
    */
   g_string_append_printf (str, 
                           "VALUES (%d, '%s - %d', TRUE, getpgusername()) ",
                           pk_num, _("New Project"), pk_num);
   
   res = PQexec (conn, str->str);
   if (!chk_sql_error (res, _("Create new project")))
      {
      pk_num = INVALID_PK;
      }

   finalize_transaction (conn, res);
   PQclear (res);
   g_string_free (str, TRUE);

   return pk_num;
}


static void 
on_add_project (GtkWidget *widget, gpointer user_data)
{
   GtkWidget   *dlg;
   PGconn      *conn;
   gboolean     admin_user;
   GList       *child_list;
   gint         pk_num;

   conn = g_object_get_data (G_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   admin_user = (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (user_data),
                                                     ADMIN_USER)) == 1);
   
   pk_num = create_new_project (conn);
   if (pk_num != INVALID_PK)
      {
      dlg = create_project_edit_dlg (pk_num, conn,
                                     GTK_WIDGET (user_data), admin_user);
      gtk_widget_show (dlg);

      /*
       * Setup the child_dead() callback to remove the child from the
       * child_list when the child dies.
       */
      g_signal_connect (dlg, "destroy", G_CALLBACK (child_dead),
                        (gpointer)my_child_list);

      child_list = g_object_get_data (G_OBJECT (user_data), my_child_list);
      child_list = g_list_append (child_list, dlg);
      g_object_set_data (G_OBJECT (user_data), my_child_list, child_list);
      }
}


static void 
on_modify_project (GtkWidget *widget, gpointer user_data)
{
   GtkTreeView      *tree;
   GtkTreeSelection *sel;
   GtkTreeIter       iter;
   GtkTreeModel     *model;
   gint              pk_num;
   PGconn           *conn;
   GtkWidget        *main_widget = GTK_WIDGET (user_data);
   gboolean          admin_user;
   GtkWidget        *msg_dlg;
   
   conn = g_object_get_data (G_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   tree = GTK_TREE_VIEW (lookup_widget (main_widget, PROJECT_LIST_VIEW));
   g_assert (tree != NULL);
   admin_user = (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (user_data),
                                                     ADMIN_USER)) == 1);

   sel = gtk_tree_view_get_selection (tree);
   
   if (gtk_tree_selection_get_selected (sel, &model, &iter))
      {
      gtk_tree_model_get (model, &iter, PROJECT_PK, &pk_num, -1);
      raise_or_create_project_edit_dlg (main_widget, pk_num,
                                        conn, admin_user);
      }
   else
      {
      msg_dlg = gtk_message_dialog_new
         (GTK_WINDOW (user_data), 0,
          GTK_MESSAGE_INFO,
          GTK_BUTTONS_OK,
          _("To modify a project, you must select a project to modify."));
      gtk_dialog_run (GTK_DIALOG (msg_dlg));
      gtk_widget_destroy (msg_dlg);
      }
}


static void 
on_delete_project (GtkWidget *widget, gpointer user_data)
{
   GtkTreeView      *tree;
   GtkTreeSelection *sel;
   GtkTreeIter       iter;
   GtkTreeModel     *model;
   gint              pk_num;
   PGconn           *conn;
   GtkWidget        *main_widget = GTK_WIDGET (user_data);
   GtkWidget        *main_win;
   GtkWidget        *msg_dlg;
   
   conn = g_object_get_data (G_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   tree = GTK_TREE_VIEW (lookup_widget (main_widget, PROJECT_LIST_VIEW));
   g_assert (tree != NULL);
   main_win = lookup_widget (main_widget, MY_PARENT);
   g_assert (main_win != NULL);

   sel = gtk_tree_view_get_selection (tree);
   
   if (gtk_tree_selection_get_selected (sel, &model, &iter))
      {
      msg_dlg = gtk_message_dialog_new
         (GTK_WINDOW (user_data), 0,
          GTK_MESSAGE_QUESTION,
          GTK_BUTTONS_YES_NO,
          _("Deleting the project will remove all associated PRs.\n"
            "This action is not reversable.\n"
            "Are you sure you want to continue?"));
      gtk_dialog_set_default_response (GTK_DIALOG (msg_dlg), GTK_RESPONSE_NO);
      if (gtk_dialog_run (GTK_DIALOG (msg_dlg)) == GTK_RESPONSE_YES)
         {
         gtk_tree_model_get (model, &iter, PROJECT_PK, &pk_num, -1);
         delete_project (conn, pk_num);
         /*
          * the project list dialog and the project list on the main
          * window both need re-drawing....
          */
         draw_project_list (GTK_WINDOW (main_widget), conn);
         refresh_project_list (main_win);
         }
      gtk_widget_destroy (msg_dlg);
      }
   else
      {
      msg_dlg = gtk_message_dialog_new
         (GTK_WINDOW (user_data), 0,
          GTK_MESSAGE_INFO,
          GTK_BUTTONS_OK,
          _("To delete a project, you must select a project to delete."));
      gtk_dialog_run (GTK_DIALOG (msg_dlg));
      gtk_widget_destroy (msg_dlg);
      }
}


static void 
on_show_inactive_cb_toggled (GtkToggleButton  *togglebutton,
                             gpointer          user_data)
{
   PGconn     *conn;
   
   conn = g_object_get_data (G_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);

   draw_project_list (GTK_WINDOW (user_data), conn);
}


static gboolean
on_button_press_event (GtkWidget      *widget,
                       GdkEventButton *event,
                       gpointer        user_data)
{
   static GtkMenu   *popup = NULL;
   static GtkWidget *new_item;
   static GtkWidget *mod_item;
   static GtkWidget *del_item;
   GtkTreeModel     *model;
   GtkTreeSelection *sel;
   GtkTreeIter       iter;
   GtkTreePath      *path;
   GtkWidget        *main_win = GTK_WIDGET (user_data);
   gboolean          admin_user;
   PGconn           *conn;
   gint              pk_num;
   gboolean          handled = FALSE;

   if (popup == NULL)
      {
      popup = GTK_MENU (gtk_menu_new());
      gtk_widget_show (GTK_WIDGET (popup));

      new_item = gtk_menu_item_new_with_label (_("New Project"));
      g_signal_connect (new_item, "activate",
                        G_CALLBACK (on_add_project), user_data);
      gtk_menu_shell_append (GTK_MENU_SHELL (popup), new_item);
      gtk_widget_show (new_item);

      mod_item = gtk_menu_item_new_with_label (_("Open Project"));
      g_signal_connect (mod_item, "activate",
                        G_CALLBACK (on_modify_project), user_data);
      gtk_menu_shell_append (GTK_MENU_SHELL (popup), mod_item);
      gtk_widget_show (mod_item);

      del_item = gtk_menu_item_new_with_label (_("Delete Project"));
      g_signal_connect (del_item, "activate",
                        G_CALLBACK (on_delete_project), user_data);
      gtk_menu_shell_append (GTK_MENU_SHELL (popup), del_item);
      gtk_widget_show (del_item);
      }

   admin_user = (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (main_win),
                                                     ADMIN_USER)) == 1);
   conn = g_object_get_data (G_OBJECT (main_win), MY_DB_CONN);
   g_assert (conn != NULL);

   if (event)
      {
      switch (event->button)
         {
         case 1:
            switch (event->type)
               {
               case GDK_2BUTTON_PRESS:
                  handled = TRUE;
                  sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
                  if (gtk_tree_selection_get_selected (sel, &model, &iter))
                     {
                     gtk_tree_model_get (model, &iter,
                                         PROJECT_PK, &pk_num, -1);
                     raise_or_create_project_edit_dlg (main_win,
                                                       pk_num, conn,
                                                       admin_user);
                     }
                  break;
               default:
                  break;
               }
            break;
         case 2:
            break;
         case 3:
            switch (event->type)
               {
               case GDK_BUTTON_PRESS:
                  handled = TRUE;
                  if (gtk_tree_view_get_path_at_pos
                        (GTK_TREE_VIEW (widget), event->x, event->y,
                         &path, NULL, NULL, NULL))
                     {
                     sel = gtk_tree_view_get_selection
                        (GTK_TREE_VIEW (widget));
                     gtk_tree_selection_select_path (sel, path);
                     gtk_widget_set_sensitive (new_item, admin_user);
                     gtk_widget_set_sensitive (del_item, admin_user);
                     gtk_menu_popup (popup, NULL, NULL, NULL, NULL,
                                     event->button, event->time);
                     gtk_tree_path_free (path);
                     }
                  break;
               default:
                  break;
               }
         default:
            break;
         }
      }

   return handled;
}


static void
update_project_active (GtkCellRendererToggle *cell,
                       gchar                 *path_str,
                       gpointer               data)
{
   GtkTreeView   *tree;
   GtkTreeModel  *model;
   GtkTreePath   *path = gtk_tree_path_new_from_string (path_str);
   GtkTreeIter    iter;
   gint           pk;
   PGconn        *conn;
   PGresult      *res;
   GString       *sql;
   gboolean       active;
   gboolean       editable;

   tree = GTK_TREE_VIEW (lookup_widget (GTK_WIDGET (data), PROJECT_LIST_VIEW));
   g_assert (tree != NULL);
   model = gtk_tree_view_get_model (tree);
   conn = g_object_get_data (G_OBJECT (data), MY_DB_CONN);
   g_assert (conn != NULL);

   gtk_tree_model_get_iter (model, &iter, path);
   gtk_tree_model_get (model, &iter, PROJECT_PK, &pk,
                       PROJECT_ACTIVE, &active,
                       PROJECT_EDITABLE, &editable, -1);

   if (editable)
      {
      active = !active;
      sql = g_string_new ("update project set active = ");
      g_string_append_printf (sql, "%s where project_num = %d",
                              active ? "TRUE" : "FALSE", pk);
      res = PQexec (conn, sql->str);
      if (chk_sql_error (res, _("Update Project Active Status")))
         {
         gtk_list_store_set (GTK_LIST_STORE (model), &iter,
                             PROJECT_ACTIVE, active, -1);
         }
      PQclear (res);
      g_string_free (sql, TRUE);
      }

   gtk_tree_path_free (path);
}

static GtkWidget* 
create_project_dlg (GtkWidget *parent, PGconn  *conn, gboolean user_is_admin)
{
  GtkWidget         *project_dlg;
  GtkWidget         *dialog_vbox15;
  GtkWidget         *vbox29;
  GtkWidget         *project_list_view;
  GtkListStore      *project_list_store;
  GtkWidget         *hbox33;
  GtkWidget         *add_button;
  GtkWidget         *modify_button;
  GtkWidget         *delete_button;
  GtkWidget         *show_inactive_cb;
  GtkWidget         *scrolledwindow;
  GtkWidget         *warn_hbox;
  GtkWidget         *warn_icon;
  GtkWidget         *warn_lbl;
  GtkTreeViewColumn *col;
  GtkCellRenderer   *rend;
  GtkTooltips       *tooltips;
  GConfClient       *client;

  tooltips = g_object_get_data (G_OBJECT (parent), TOOLTIPS);
  g_assert (tooltips != NULL);
  client = g_object_get_data (G_OBJECT (parent), MY_GCONF_CLIENT);
  g_assert (client != NULL);

  project_dlg = gtk_dialog_new_with_buttons
     (_("Projects"),
      GTK_WINDOW (parent),
      GTK_DIALOG_DESTROY_WITH_PARENT,
      GTK_STOCK_CLOSE,
      GTK_RESPONSE_CLOSE,
      NULL);
  GLADE_HOOKUP_OBJECT_NO_REF (project_dlg, project_dlg, PROJECT_DIALOG);
  gtk_container_set_border_width (GTK_CONTAINER (project_dlg), 5);
  g_signal_connect (project_dlg, "show",
                    G_CALLBACK (on_project_dlg_show), NULL);
  g_signal_connect (project_dlg, "destroy",
                    G_CALLBACK (on_project_dlg_destroy), NULL);
  g_signal_connect (project_dlg, "response",
                    G_CALLBACK (on_project_dlg_clicked), NULL);
  gtk_window_set_resizable (GTK_WINDOW (project_dlg), TRUE);
  set_dlg_size (client, project_dlg, PRJ_DLG_SEC, 550, 265);

  g_object_set_data (G_OBJECT (project_dlg), TOP_DOG, project_dlg);
  g_object_set_data (G_OBJECT (project_dlg), MY_PARENT, parent);
  g_object_set_data (G_OBJECT (project_dlg), MY_DB_CONN, conn);
  g_object_set_data (G_OBJECT (project_dlg), MY_GCONF_CLIENT, client);
  g_object_set_data (G_OBJECT (project_dlg), ADMIN_USER,
        GINT_TO_POINTER (user_is_admin ? 1 : 0));

  dialog_vbox15 = GTK_DIALOG (project_dlg)->vbox;
  GLADE_HOOKUP_OBJECT (project_dlg, dialog_vbox15, "dialog_vbox15");
  gtk_box_set_spacing (GTK_BOX (dialog_vbox15), STD_BOX_SPACING);
  gtk_widget_show (dialog_vbox15);

  vbox29 = gtk_vbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_dlg, vbox29, "vbox29");
  gtk_widget_show (vbox29);
  gtk_box_pack_start (GTK_BOX (dialog_vbox15), vbox29, TRUE, TRUE, 0);

  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  GLADE_HOOKUP_OBJECT (project_dlg, scrolledwindow, "scrolledwindow");
  gtk_widget_show (scrolledwindow);
  gtk_box_pack_start (GTK_BOX (vbox29), scrolledwindow, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow),
                                       GTK_SHADOW_ETCHED_IN);

  project_list_store = gtk_list_store_new (NUM_PROJECT_COLS,
                                           G_TYPE_STRING,
                                           G_TYPE_STRING,
                                           G_TYPE_BOOLEAN,
                                           G_TYPE_BOOLEAN,
                                           G_TYPE_INT);
  project_list_view = gtk_tree_view_new_with_model
     (GTK_TREE_MODEL (project_list_store));
  GLADE_HOOKUP_OBJECT (project_dlg, project_list_view, PROJECT_LIST_VIEW);
  gtk_widget_show (project_list_view);
  gtk_container_add (GTK_CONTAINER (scrolledwindow), project_list_view);
  g_signal_connect (project_list_view, "button_press_event",
                    G_CALLBACK (on_button_press_event), project_dlg);

  rend = gtk_cell_renderer_toggle_new ();
  col = gtk_tree_view_column_new_with_attributes (_("Active"), rend,
                                                  "active", PROJECT_ACTIVE,
                                                  NULL);
  g_signal_connect (rend, "toggled", G_CALLBACK (update_project_active),
                    project_dlg);
  gtk_tree_view_append_column (GTK_TREE_VIEW (project_list_view), col);

  rend = gtk_cell_renderer_text_new ();
  col = gtk_tree_view_column_new_with_attributes (_("Project Name"), rend,
                                                  "text", PROJECT_NAME,
                                                  NULL);
  g_object_set_data (G_OBJECT (rend), COLUMN_NUM,
                     GINT_TO_POINTER (PROJECT_NAME));
  gtk_tree_view_append_column (GTK_TREE_VIEW (project_list_view), col);

  rend = gtk_cell_renderer_text_new ();
  col = gtk_tree_view_column_new_with_attributes (_("Description"), rend,
                                                  "text", PROJECT_DESCR, 
                                                  NULL);
  g_object_set_data (G_OBJECT (rend), COLUMN_NUM, 
                     GINT_TO_POINTER (PROJECT_DESCR));
  gtk_tree_view_append_column (GTK_TREE_VIEW (project_list_view), col);

  hbox33 = gtk_hbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (project_dlg, hbox33, "hbox33");
  gtk_widget_show (hbox33);
  gtk_box_pack_start (GTK_BOX (vbox29), hbox33, FALSE, FALSE, 0);

  add_button = gtk_button_new_from_stock (GTK_STOCK_ADD);
  GLADE_HOOKUP_OBJECT (project_dlg, add_button, ADD_BUTTON);
  gtk_widget_show (add_button);
  gtk_box_pack_start (GTK_BOX (hbox33), add_button, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, add_button, _("Add a project."), NULL);
  g_signal_connect (add_button, "clicked",
                    G_CALLBACK (on_add_project), project_dlg);
  gtk_widget_set_sensitive (add_button, user_is_admin);

  modify_button = gtk_button_new_from_stock (GTK_STOCK_OPEN);
  GLADE_HOOKUP_OBJECT (project_dlg, modify_button, MOD_BUTTON);
  gtk_widget_show (modify_button);
  gtk_box_pack_start (GTK_BOX (hbox33), modify_button, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, 
                        modify_button, 
                        _("Modify an existing project."), 
                        NULL);
  g_signal_connect (modify_button, "clicked",
                    G_CALLBACK (on_modify_project), project_dlg);

  delete_button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
  GLADE_HOOKUP_OBJECT (project_dlg, delete_button, DEL_BUTTON);
  gtk_widget_show (delete_button);
  gtk_box_pack_start (GTK_BOX (hbox33), delete_button, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, delete_button, _("Delete an existing project.  If the project is referenced in problem report, the project cannot be deleted, but can be made inactive."), NULL);
  g_signal_connect (delete_button, "clicked",
                    G_CALLBACK (on_delete_project), project_dlg);
  gtk_widget_set_sensitive (delete_button, user_is_admin);

  show_inactive_cb = gtk_check_button_new_with_label
     (_("Show Inactive Projects"));
  GLADE_HOOKUP_OBJECT (project_dlg, show_inactive_cb, SHOW_INACTIVE_CB);
  gtk_widget_show (show_inactive_cb);
  gtk_box_pack_start (GTK_BOX (hbox33), show_inactive_cb, TRUE, TRUE, 0);
  g_signal_connect (show_inactive_cb, "toggled",
                    G_CALLBACK (on_show_inactive_cb_toggled), project_dlg);

  if (!user_is_admin)
     {
     warn_hbox = gtk_hbox_new (FALSE, STD_BOX_SPACING);
     GLADE_HOOKUP_OBJECT (project_dlg, warn_hbox, "warn1_hbox");
     gtk_box_pack_start (GTK_BOX (dialog_vbox15), warn_hbox, FALSE, FALSE, 0);
     gtk_widget_show (warn_hbox);

     warn_icon = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING,
                                           GTK_ICON_SIZE_SMALL_TOOLBAR);
     GLADE_HOOKUP_OBJECT (project_dlg, warn_icon, "warn1_icon");
     gtk_box_pack_start (GTK_BOX (warn_hbox), warn_icon, FALSE, FALSE, 0);
     gtk_widget_show (warn_icon);

     warn_lbl = gtk_label_new (NULL);
     GLADE_HOOKUP_OBJECT (project_dlg, warn_lbl, "warn1_lbl");
     gtk_label_set_markup
        (GTK_LABEL (warn_lbl),
         _("<span weight=\"bold\">Only administrators can add and remove projects</span>."));
     gtk_box_pack_start (GTK_BOX (warn_hbox), warn_lbl, FALSE, FALSE, 0);
     gtk_widget_show (warn_lbl);
     }

  g_object_set_data (G_OBJECT (project_dlg), TOOLTIPS, tooltips);

  return project_dlg;
}


void
destroy_project_dlg (void)
{
   if (project_dlg != NULL)
      {
      gtk_widget_destroy (project_dlg);
      project_dlg = NULL;
      }
}


void
show_project_dlg (GtkWidget *parent, PGconn *conn, gboolean user_is_admin)
{
   /*
    * If the dialog box exists, just make sure it is visable, otherwise
    * create it.
    */
   if (project_dlg)
      {
      gdk_window_show (project_dlg->window);
      gdk_window_raise (project_dlg->window);
      }
   else
      {
      project_dlg = create_project_dlg (parent, conn, user_is_admin);

      /*
       * When the dialog is destroyed, call gtk_widget_destroyed(),
       * which will set the "user_data" (&project_dlg in this case) to
       * NULL.
       */
      g_signal_connect (project_dlg, "destroy",
                        G_CALLBACK (gtk_widget_destroyed), &project_dlg);
      gtk_widget_show (project_dlg);
      }
}


static gboolean
edit_dlg_dirty (GtkWidget *dlg)
{
   GtkTextView   *view;
   GtkTextBuffer *buf;

   view = GTK_TEXT_VIEW (lookup_widget (dlg, DESCR_TEXT));
   buf = gtk_text_view_get_buffer (view);

   if (gtk_text_buffer_get_modified (buf))
      {
      return TRUE;
      }

   return objects_children_are_dirty (G_OBJECT (dlg));
}


gboolean
project_dlg_dirty (void)
{
   GList *list;

   if (project_dlg == NULL)
      {
      return FALSE;
      }

   list = g_object_get_data (G_OBJECT (project_dlg), my_child_list);
   list = g_list_first (list);
   while (list != NULL)
      {
      if (edit_dlg_dirty (GTK_WIDGET (list->data)))
         {
         return TRUE;
         }
      list = list->next;
      }

   return FALSE;
}
